From a0faf5691823f50f0be0d76550a9458618afaa65 Mon Sep 17 00:00:00 2001
From: Michael Paquier <michael@paquier.xyz>
Date: Fri, 4 Aug 2017 16:20:54 +0200
Subject: [PATCH 4/4] Eliminate user-visible cache lookup errors for objaddr
 SQL functions

When using the following functions, users could see various types of
errors like "cache lookup failed for OID XXX":
* pg_describe_object
* pg_identify_object_as_address
* pg_identify_object
All the lower set of APIs managing object addresses for all types of
the system are made smarter by gaining a missing_ok argument that allows
any caller to control if he is willing to have an actual error or if
he wants to control error at its level by having empty objects if they
are undefined.

Regression tests added in this commit failed with such errors before
being patched.
---
 contrib/sepgsql/database.c                   |   6 +-
 contrib/sepgsql/dml.c                        |   4 +-
 contrib/sepgsql/label.c                      |   4 +-
 contrib/sepgsql/proc.c                       |  14 +-
 contrib/sepgsql/relation.c                   |  20 +-
 contrib/sepgsql/schema.c                     |   6 +-
 doc/src/sgml/func.sgml                       |   7 +-
 src/backend/catalog/dependency.c             |  19 +-
 src/backend/catalog/objectaddress.c          | 824 ++++++++++++++++++++-------
 src/backend/catalog/pg_depend.c              |   4 +-
 src/backend/catalog/pg_shdepend.c            |   8 +-
 src/backend/commands/event_trigger.c         |   9 +-
 src/backend/commands/extension.c             |   6 +-
 src/backend/commands/tablecmds.c             |  10 +-
 src/backend/utils/adt/format_type.c          |  23 +-
 src/backend/utils/adt/regproc.c              |  16 +-
 src/include/catalog/objectaddress.h          |  12 +-
 src/include/utils/builtins.h                 |   3 +
 src/include/utils/regproc.h                  |   4 +-
 src/test/regress/expected/object_address.out | 708 +++++++++++++++++++++++
 src/test/regress/sql/object_address.sql      | 124 ++++
 21 files changed, 1541 insertions(+), 290 deletions(-)

diff --git a/contrib/sepgsql/database.c b/contrib/sepgsql/database.c
index 8fc5a87e00..0acb189571 100644
--- a/contrib/sepgsql/database.c
+++ b/contrib/sepgsql/database.c
@@ -142,7 +142,7 @@ sepgsql_database_drop(Oid databaseId)
 	object.classId = DatabaseRelationId;
 	object.objectId = databaseId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_DATABASE,
@@ -169,7 +169,7 @@ sepgsql_database_setattr(Oid databaseId)
 	object.classId = DatabaseRelationId;
 	object.objectId = databaseId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_DATABASE,
@@ -193,7 +193,7 @@ sepgsql_database_relabel(Oid databaseId, const char *seclabel)
 	object.classId = DatabaseRelationId;
 	object.objectId = databaseId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	/*
 	 * check db_database:{setattr relabelfrom} permission
diff --git a/contrib/sepgsql/dml.c b/contrib/sepgsql/dml.c
index b9959b54d4..1bf3208214 100644
--- a/contrib/sepgsql/dml.c
+++ b/contrib/sepgsql/dml.c
@@ -186,7 +186,7 @@ check_relation_privileges(Oid relOid,
 	object.classId = RelationRelationId;
 	object.objectId = relOid;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 	switch (relkind)
 	{
 		case RELKIND_RELATION:
@@ -263,7 +263,7 @@ check_relation_privileges(Oid relOid,
 		object.classId = RelationRelationId;
 		object.objectId = relOid;
 		object.objectSubId = attnum;
-		audit_name = getObjectDescription(&object);
+		audit_name = getObjectDescription(&object, false);
 
 		result = sepgsql_avc_check_perms(&object,
 										 SEPG_CLASS_DB_COLUMN,
diff --git a/contrib/sepgsql/label.c b/contrib/sepgsql/label.c
index cbb9249be7..af569d24e4 100644
--- a/contrib/sepgsql/label.c
+++ b/contrib/sepgsql/label.c
@@ -372,7 +372,7 @@ sepgsql_fmgr_hook(FmgrHookEventType event,
 					sepgsql_avc_check_perms(&object,
 											SEPG_CLASS_DB_PROCEDURE,
 											SEPG_DB_PROCEDURE__ENTRYPOINT,
-											getObjectDescription(&object),
+											getObjectDescription(&object, false),
 											true);
 
 					sepgsql_avc_check_perms_label(stack->new_label,
@@ -543,7 +543,7 @@ sepgsql_object_relabel(const ObjectAddress *object, const char *seclabel)
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 					 errmsg("sepgsql provider does not support labels on %s",
-							getObjectTypeDescription(object))));
+							getObjectTypeDescription(object, false))));
 			break;
 	}
 }
diff --git a/contrib/sepgsql/proc.c b/contrib/sepgsql/proc.c
index 14faa5fac6..c7bd656c62 100644
--- a/contrib/sepgsql/proc.c
+++ b/contrib/sepgsql/proc.c
@@ -81,7 +81,7 @@ sepgsql_proc_post_create(Oid functionId)
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_SCHEMA,
 							SEPG_DB_SCHEMA__ADD_NAME,
-							getObjectIdentity(&object),
+							getObjectIdentity(&object, false),
 							true);
 
 	/*
@@ -115,7 +115,7 @@ sepgsql_proc_post_create(Oid functionId)
 		object.classId = TypeRelationId;
 		object.objectId = proForm->proargtypes.values[i];
 		object.objectSubId = 0;
-		appendStringInfoString(&audit_name, getObjectIdentity(&object));
+		appendStringInfoString(&audit_name, getObjectIdentity(&object, false));
 	}
 	appendStringInfoChar(&audit_name, ')');
 
@@ -165,7 +165,7 @@ sepgsql_proc_drop(Oid functionId)
 	object.classId = NamespaceRelationId;
 	object.objectId = get_func_namespace(functionId);
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_SCHEMA,
@@ -180,7 +180,7 @@ sepgsql_proc_drop(Oid functionId)
 	object.classId = ProcedureRelationId;
 	object.objectId = functionId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_PROCEDURE,
@@ -205,7 +205,7 @@ sepgsql_proc_relabel(Oid functionId, const char *seclabel)
 	object.classId = ProcedureRelationId;
 	object.objectId = functionId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	/*
 	 * check db_procedure:{setattr relabelfrom} permission
@@ -293,7 +293,7 @@ sepgsql_proc_setattr(Oid functionId)
 	object.classId = ProcedureRelationId;
 	object.objectId = functionId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_PROCEDURE,
@@ -325,7 +325,7 @@ sepgsql_proc_execute(Oid functionId)
 	object.classId = ProcedureRelationId;
 	object.objectId = functionId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_PROCEDURE,
 							SEPG_DB_PROCEDURE__EXECUTE,
diff --git a/contrib/sepgsql/relation.c b/contrib/sepgsql/relation.c
index 228869a520..bb3ed26a00 100644
--- a/contrib/sepgsql/relation.c
+++ b/contrib/sepgsql/relation.c
@@ -103,7 +103,7 @@ sepgsql_attribute_post_create(Oid relOid, AttrNumber attnum)
 
 	initStringInfo(&audit_name);
 	appendStringInfo(&audit_name, "%s.%s",
-					 getObjectIdentity(&object),
+					 getObjectIdentity(&object, false),
 					 quote_identifier(NameStr(attForm->attname)));
 	sepgsql_avc_check_perms_label(ncontext,
 								  SEPG_CLASS_DB_COLUMN,
@@ -147,7 +147,7 @@ sepgsql_attribute_drop(Oid relOid, AttrNumber attnum)
 	object.classId = RelationRelationId;
 	object.objectId = relOid;
 	object.objectSubId = attnum;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_COLUMN,
@@ -179,7 +179,7 @@ sepgsql_attribute_relabel(Oid relOid, AttrNumber attnum,
 	object.classId = RelationRelationId;
 	object.objectId = relOid;
 	object.objectSubId = attnum;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	/*
 	 * check db_column:{setattr relabelfrom} permission
@@ -223,7 +223,7 @@ sepgsql_attribute_setattr(Oid relOid, AttrNumber attnum)
 	object.classId = RelationRelationId;
 	object.objectId = relOid;
 	object.objectSubId = attnum;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_COLUMN,
@@ -289,7 +289,7 @@ sepgsql_relation_post_create(Oid relOid)
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_SCHEMA,
 							SEPG_DB_SCHEMA__ADD_NAME,
-							getObjectIdentity(&object),
+							getObjectIdentity(&object, false),
 							true);
 
 	switch (classForm->relkind)
@@ -451,7 +451,7 @@ sepgsql_relation_drop(Oid relOid)
 	object.classId = NamespaceRelationId;
 	object.objectId = get_rel_namespace(relOid);
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_SCHEMA,
@@ -473,7 +473,7 @@ sepgsql_relation_drop(Oid relOid)
 	object.classId = RelationRelationId;
 	object.objectId = relOid;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							tclass,
@@ -504,7 +504,7 @@ sepgsql_relation_drop(Oid relOid)
 			object.classId = RelationRelationId;
 			object.objectId = relOid;
 			object.objectSubId = attForm->attnum;
-			audit_name = getObjectIdentity(&object);
+			audit_name = getObjectIdentity(&object, false);
 
 			sepgsql_avc_check_perms(&object,
 									SEPG_CLASS_DB_COLUMN,
@@ -545,7 +545,7 @@ sepgsql_relation_relabel(Oid relOid, const char *seclabel)
 	object.classId = RelationRelationId;
 	object.objectId = relOid;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	/*
 	 * check db_xxx:{setattr relabelfrom} permission
@@ -656,7 +656,7 @@ sepgsql_relation_setattr(Oid relOid)
 	object.classId = RelationRelationId;
 	object.objectId = relOid;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							tclass,
diff --git a/contrib/sepgsql/schema.c b/contrib/sepgsql/schema.c
index d418577b75..6c2c12ef63 100644
--- a/contrib/sepgsql/schema.c
+++ b/contrib/sepgsql/schema.c
@@ -124,7 +124,7 @@ sepgsql_schema_drop(Oid namespaceId)
 	object.classId = NamespaceRelationId;
 	object.objectId = namespaceId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	sepgsql_avc_check_perms(&object,
 							SEPG_CLASS_DB_SCHEMA,
@@ -149,7 +149,7 @@ sepgsql_schema_relabel(Oid namespaceId, const char *seclabel)
 	object.classId = NamespaceRelationId;
 	object.objectId = namespaceId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	/*
 	 * check db_schema:{setattr relabelfrom} permission
@@ -187,7 +187,7 @@ check_schema_perms(Oid namespaceId, uint32 required, bool abort_on_violation)
 	object.classId = NamespaceRelationId;
 	object.objectId = namespaceId;
 	object.objectSubId = 0;
-	audit_name = getObjectIdentity(&object);
+	audit_name = getObjectIdentity(&object, false);
 
 	result = sepgsql_avc_check_perms(&object,
 									 SEPG_CLASS_DB_SCHEMA,
diff --git a/doc/src/sgml/func.sgml b/doc/src/sgml/func.sgml
index 4dd9d029e6..c635f3b522 100644
--- a/doc/src/sgml/func.sgml
+++ b/doc/src/sgml/func.sgml
@@ -17538,7 +17538,8 @@ SELECT collation for ('foo' COLLATE "de_DE");
    This description is intended to be human-readable, and might be translated,
    depending on server configuration.
    This is useful to determine the identity of an object as stored in the
-   <structname>pg_depend</structname> catalog.
+   <structname>pg_depend</structname> catalog. This function returns
+   <literal>NULL</> values for undefined objects.
   </para>
 
   <para>
@@ -17554,7 +17555,8 @@ SELECT collation for ('foo' COLLATE "de_DE");
    identifier of the object, otherwise <literal>NULL</literal>;
    <parameter>identity</parameter> is the complete object identity, with the precise format
    depending on object type, and each part within the format being
-   schema-qualified and quoted as necessary.
+   schema-qualified and quoted as necessary. Undefined objects are identified
+   with <literal>NULL</> values.
   </para>
 
   <para>
@@ -17569,6 +17571,7 @@ SELECT collation for ('foo' COLLATE "de_DE");
    <function>pg_get_object_address</function> to obtain the internal address
    of the object.
    This function is the inverse of <function>pg_get_object_address</function>.
+   Undefined objects are identified with <literal>NULL</> values.
   </para>
 
   <para>
diff --git a/src/backend/catalog/dependency.c b/src/backend/catalog/dependency.c
index 033c4358ea..ccefc8b06d 100644
--- a/src/backend/catalog/dependency.c
+++ b/src/backend/catalog/dependency.c
@@ -607,11 +607,11 @@ findDependentObjects(const ObjectAddress *object,
 						ReleaseDeletionLock(object);
 						return;
 					}
-					otherObjDesc = getObjectDescription(&otherObject);
+					otherObjDesc = getObjectDescription(&otherObject, false);
 					ereport(ERROR,
 							(errcode(ERRCODE_DEPENDENT_OBJECTS_STILL_EXIST),
 							 errmsg("cannot drop %s because %s requires it",
-									getObjectDescription(object),
+									getObjectDescription(object, false),
 									otherObjDesc),
 							 errhint("You can drop %s instead.",
 									 otherObjDesc)));
@@ -682,11 +682,11 @@ findDependentObjects(const ObjectAddress *object,
 				 * the depender fields...
 				 */
 				elog(ERROR, "incorrect use of PIN dependency with %s",
-					 getObjectDescription(object));
+					 getObjectDescription(object, false));
 				break;
 			default:
 				elog(ERROR, "unrecognized dependency type '%c' for %s",
-					 foundDep->deptype, getObjectDescription(object));
+					 foundDep->deptype, getObjectDescription(object, false));
 				break;
 		}
 	}
@@ -777,12 +777,12 @@ findDependentObjects(const ObjectAddress *object,
 				ereport(ERROR,
 						(errcode(ERRCODE_DEPENDENT_OBJECTS_STILL_EXIST),
 						 errmsg("cannot drop %s because it is required by the database system",
-								getObjectDescription(object))));
+								getObjectDescription(object, false))));
 				subflags = 0;	/* keep compiler quiet */
 				break;
 			default:
 				elog(ERROR, "unrecognized dependency type '%c' for %s",
-					 foundDep->deptype, getObjectDescription(object));
+					 foundDep->deptype, getObjectDescription(object, false));
 				subflags = 0;	/* keep compiler quiet */
 				break;
 		}
@@ -876,7 +876,7 @@ reportDependentObjects(const ObjectAddresses *targetObjects,
 		if (extra->flags & DEPFLAG_ORIGINAL)
 			continue;
 
-		objDesc = getObjectDescription(obj);
+		objDesc = getObjectDescription(obj, false);
 
 		/*
 		 * If, at any stage of the recursive search, we reached the object via
@@ -899,7 +899,8 @@ reportDependentObjects(const ObjectAddresses *targetObjects,
 		}
 		else if (behavior == DROP_RESTRICT)
 		{
-			char	   *otherDesc = getObjectDescription(&extra->dependee);
+			char	   *otherDesc = getObjectDescription(&extra->dependee,
+														 false);
 
 			if (numReportedClient < MAX_REPORTED_DEPS)
 			{
@@ -957,7 +958,7 @@ reportDependentObjects(const ObjectAddresses *targetObjects,
 			ereport(ERROR,
 					(errcode(ERRCODE_DEPENDENT_OBJECTS_STILL_EXIST),
 					 errmsg("cannot drop %s because other objects depend on it",
-							getObjectDescription(origObject)),
+							getObjectDescription(origObject, false)),
 					 errdetail("%s", clientdetail.data),
 					 errdetail_log("%s", logdetail.data),
 					 errhint("Use DROP ... CASCADE to drop the dependent objects too.")));
diff --git a/src/backend/catalog/objectaddress.c b/src/backend/catalog/objectaddress.c
index 6ce8f58fbe..38bb52f93f 100644
--- a/src/backend/catalog/objectaddress.c
+++ b/src/backend/catalog/objectaddress.c
@@ -751,14 +751,20 @@ static ObjectAddress get_object_address_defacl(List *object,
 						  bool missing_ok);
 static const ObjectPropertyType *get_object_property_data(Oid class_id);
 
-static void getRelationDescription(StringInfo buffer, Oid relid);
-static void getOpFamilyDescription(StringInfo buffer, Oid opfid);
+static void getRelationDescription(StringInfo buffer, Oid relid,
+								   bool missing_ok);
+static void getOpFamilyDescription(StringInfo buffer, Oid opfid,
+								   bool missing_ok);
 static void getRelationTypeDescription(StringInfo buffer, Oid relid,
-						   int32 objectSubId);
-static void getProcedureTypeDescription(StringInfo buffer, Oid procid);
-static void getConstraintTypeDescription(StringInfo buffer, Oid constroid);
-static void getOpFamilyIdentity(StringInfo buffer, Oid opfid, List **object);
-static void getRelationIdentity(StringInfo buffer, Oid relid, List **object);
+									   int32 objectSubId, bool missing_ok);
+static void getProcedureTypeDescription(StringInfo buffer, Oid procid,
+										bool missing_ok);
+static void getConstraintTypeDescription(StringInfo buffer, Oid constroid,
+										 bool missing_ok);
+static void getOpFamilyIdentity(StringInfo buffer, Oid opfid, List **object,
+								bool missing_ok);
+static void getRelationIdentity(StringInfo buffer, Oid relid, List **object,
+								bool missing_ok);
 
 /*
  * Translate an object name and arguments (as passed by the parser) to an
@@ -1626,7 +1632,7 @@ get_object_address_opf_member(ObjectType objtype,
 										membernum,
 										TypeNameToString(typenames[0]),
 										TypeNameToString(typenames[1]),
-										getObjectDescription(&famaddr))));
+										getObjectDescription(&famaddr, false))));
 				}
 				else
 				{
@@ -1657,7 +1663,7 @@ get_object_address_opf_member(ObjectType objtype,
 										membernum,
 										TypeNameToString(typenames[0]),
 										TypeNameToString(typenames[1]),
-										getObjectDescription(&famaddr))));
+										getObjectDescription(&famaddr, false))));
 				}
 				else
 				{
@@ -2649,10 +2655,12 @@ get_catalog_object_by_oid(Relation catalog, Oid objectId)
 /*
  * getObjectDescription: build an object description for messages
  *
- * The result is a palloc'd string.
+ * The result is a palloc'd string. In the event of an undefined object,
+ * NULL is returned if missing_ok is true. If missing_ok is false an
+ * error is generated instead.
  */
 char *
-getObjectDescription(const ObjectAddress *object)
+getObjectDescription(const ObjectAddress *object, bool missing_ok)
 {
 	StringInfoData buffer;
 
@@ -2661,22 +2669,28 @@ getObjectDescription(const ObjectAddress *object)
 	switch (getObjectClass(object))
 	{
 		case OCLASS_CLASS:
-			getRelationDescription(&buffer, object->objectId);
+			getRelationDescription(&buffer, object->objectId, missing_ok);
 			if (object->objectSubId != 0)
-				appendStringInfo(&buffer, _(" column %s"),
-								 get_attname(object->objectId,
-											 object->objectSubId,
-											 false));
+			{
+				char *attname = get_attname(object->objectId,
+											object->objectSubId,
+											missing_ok);
+				if (attname)
+					appendStringInfo(&buffer, _(" column %s"), attname);
+			}
 			break;
 
 		case OCLASS_PROC:
+			/* returns numerical OID for undefined function */
 			appendStringInfo(&buffer, _("function %s"),
 							 format_procedure(object->objectId));
 			break;
 
 		case OCLASS_TYPE:
+			/* returns "???" for undefined type */
 			appendStringInfo(&buffer, _("type %s"),
-							 format_type_be(object->objectId));
+							 format_type_extended(object->objectId, -1,
+												  false, missing_ok, false));
 			break;
 
 		case OCLASS_CAST:
@@ -2699,15 +2713,19 @@ getObjectDescription(const ObjectAddress *object)
 
 				tup = systable_getnext(rcscan);
 
-				if (!HeapTupleIsValid(tup))
+				if (HeapTupleIsValid(tup))
+				{
+					castForm = (Form_pg_cast) GETSTRUCT(tup);
+
+					appendStringInfo(&buffer, _("cast from %s to %s"),
+									 format_type_be(castForm->castsource),
+									 format_type_be(castForm->casttarget));
+				}
+				else if (!missing_ok)
+				{
 					elog(ERROR, "could not find tuple for cast %u",
 						 object->objectId);
-
-				castForm = (Form_pg_cast) GETSTRUCT(tup);
-
-				appendStringInfo(&buffer, _("cast from %s to %s"),
-								 format_type_be(castForm->castsource),
-								 format_type_be(castForm->casttarget));
+				}
 
 				systable_endscan(rcscan);
 				heap_close(castDesc, AccessShareLock);
@@ -2721,13 +2739,18 @@ getObjectDescription(const ObjectAddress *object)
 
 				collTup = SearchSysCache1(COLLOID,
 										  ObjectIdGetDatum(object->objectId));
-				if (!HeapTupleIsValid(collTup))
+				if (HeapTupleIsValid(collTup))
+				{
+					coll = (Form_pg_collation) GETSTRUCT(collTup);
+					appendStringInfo(&buffer, _("collation %s"),
+									 NameStr(coll->collname));
+					ReleaseSysCache(collTup);
+				}
+				else if (!missing_ok)
+				{
 					elog(ERROR, "cache lookup failed for collation %u",
 						 object->objectId);
-				coll = (Form_pg_collation) GETSTRUCT(collTup);
-				appendStringInfo(&buffer, _("collation %s"),
-								 NameStr(coll->collname));
-				ReleaseSysCache(collTup);
+				}
 				break;
 			}
 
@@ -2739,8 +2762,14 @@ getObjectDescription(const ObjectAddress *object)
 				conTup = SearchSysCache1(CONSTROID,
 										 ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(conTup))
+				{
+					if (missing_ok)
+						break;
+
 					elog(ERROR, "cache lookup failed for constraint %u",
 						 object->objectId);
+				}
+
 				con = (Form_pg_constraint) GETSTRUCT(conTup);
 
 				if (OidIsValid(con->conrelid))
@@ -2748,7 +2777,7 @@ getObjectDescription(const ObjectAddress *object)
 					StringInfoData rel;
 
 					initStringInfo(&rel);
-					getRelationDescription(&rel, con->conrelid);
+					getRelationDescription(&rel, con->conrelid, false);
 					appendStringInfo(&buffer, _("constraint %s on %s"),
 									 NameStr(con->conname), rel.data);
 					pfree(rel.data);
@@ -2770,8 +2799,13 @@ getObjectDescription(const ObjectAddress *object)
 				conTup = SearchSysCache1(CONVOID,
 										 ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(conTup))
+				{
+					if (missing_ok)
+						break;
+
 					elog(ERROR, "cache lookup failed for conversion %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("conversion %s"),
 								 NameStr(((Form_pg_conversion) GETSTRUCT(conTup))->conname));
 				ReleaseSysCache(conTup);
@@ -2799,18 +2833,22 @@ getObjectDescription(const ObjectAddress *object)
 
 				tup = systable_getnext(adscan);
 
-				if (!HeapTupleIsValid(tup))
-					elog(ERROR, "could not find tuple for attrdef %u",
-						 object->objectId);
-
-				attrdef = (Form_pg_attrdef) GETSTRUCT(tup);
+				if (HeapTupleIsValid(tup))
+				{
+					attrdef = (Form_pg_attrdef) GETSTRUCT(tup);
 
-				colobject.classId = RelationRelationId;
-				colobject.objectId = attrdef->adrelid;
-				colobject.objectSubId = attrdef->adnum;
+					colobject.classId = RelationRelationId;
+					colobject.objectId = attrdef->adrelid;
+					colobject.objectSubId = attrdef->adnum;
 
-				appendStringInfo(&buffer, _("default for %s"),
-								 getObjectDescription(&colobject));
+					appendStringInfo(&buffer, _("default for %s"),
+									 getObjectDescription(&colobject, false));
+				}
+				else if (!missing_ok)
+				{
+					elog(ERROR, "could not find tuple for attrdef %u",
+						 object->objectId);
+				}
 
 				systable_endscan(adscan);
 				heap_close(attrdefDesc, AccessShareLock);
@@ -2818,9 +2856,15 @@ getObjectDescription(const ObjectAddress *object)
 			}
 
 		case OCLASS_LANGUAGE:
-			appendStringInfo(&buffer, _("language %s"),
-							 get_language_name(object->objectId, false));
-			break;
+			{
+				char *langname = get_language_name(object->objectId,
+												   missing_ok);
+
+				if (langname)
+					appendStringInfo(&buffer, _("language %s"),
+									 get_language_name(object->objectId, false));
+				break;
+			}
 
 		case OCLASS_LARGEOBJECT:
 			appendStringInfo(&buffer, _("large object %u"),
@@ -2828,6 +2872,7 @@ getObjectDescription(const ObjectAddress *object)
 			break;
 
 		case OCLASS_OPERATOR:
+			/* returns numerical OID for undefined operator */
 			appendStringInfo(&buffer, _("operator %s"),
 							 format_operator(object->objectId));
 			break;
@@ -2843,8 +2888,13 @@ getObjectDescription(const ObjectAddress *object)
 				opcTup = SearchSysCache1(CLAOID,
 										 ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(opcTup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for opclass %u",
 						 object->objectId);
+				}
+
 				opcForm = (Form_pg_opclass) GETSTRUCT(opcTup);
 
 				amTup = SearchSysCache1(AMOID,
@@ -2871,7 +2921,7 @@ getObjectDescription(const ObjectAddress *object)
 			}
 
 		case OCLASS_OPFAMILY:
-			getOpFamilyDescription(&buffer, object->objectId);
+			getOpFamilyDescription(&buffer, object->objectId, missing_ok);
 			break;
 
 		case OCLASS_AM:
@@ -2881,8 +2931,13 @@ getObjectDescription(const ObjectAddress *object)
 				tup = SearchSysCache1(AMOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for access method %u",
 						 object->objectId);
+				}
+
 				appendStringInfo(&buffer, _("access method %s"),
 								 NameStr(((Form_pg_am) GETSTRUCT(tup))->amname));
 				ReleaseSysCache(tup);
@@ -2911,28 +2966,32 @@ getObjectDescription(const ObjectAddress *object)
 
 				tup = systable_getnext(amscan);
 
-				if (!HeapTupleIsValid(tup))
+				if (HeapTupleIsValid(tup))
+				{
+					amopForm = (Form_pg_amop) GETSTRUCT(tup);
+
+					initStringInfo(&opfam);
+					getOpFamilyDescription(&opfam, amopForm->amopfamily, false);
+
+					/*------
+					   translator: %d is the operator strategy (a number), the
+					   first two %s's are data type names, the third %s is the
+					   description of the operator family, and the last %s is
+					   the textual form of the operator with arguments.  */
+					appendStringInfo(&buffer, _("operator %d (%s, %s) of %s: %s"),
+									 amopForm->amopstrategy,
+									 format_type_be(amopForm->amoplefttype),
+									 format_type_be(amopForm->amoprighttype),
+									 opfam.data,
+									 format_operator(amopForm->amopopr));
+
+					pfree(opfam.data);
+				}
+				else if (!missing_ok)
+				{
 					elog(ERROR, "could not find tuple for amop entry %u",
 						 object->objectId);
-
-				amopForm = (Form_pg_amop) GETSTRUCT(tup);
-
-				initStringInfo(&opfam);
-				getOpFamilyDescription(&opfam, amopForm->amopfamily);
-
-				/*------
-				   translator: %d is the operator strategy (a number), the
-				   first two %s's are data type names, the third %s is the
-				   description of the operator family, and the last %s is the
-				   textual form of the operator with arguments.  */
-				appendStringInfo(&buffer, _("operator %d (%s, %s) of %s: %s"),
-								 amopForm->amopstrategy,
-								 format_type_be(amopForm->amoplefttype),
-								 format_type_be(amopForm->amoprighttype),
-								 opfam.data,
-								 format_operator(amopForm->amopopr));
-
-				pfree(opfam.data);
+				}
 
 				systable_endscan(amscan);
 				heap_close(amopDesc, AccessShareLock);
@@ -2961,28 +3020,32 @@ getObjectDescription(const ObjectAddress *object)
 
 				tup = systable_getnext(amscan);
 
-				if (!HeapTupleIsValid(tup))
+				if (HeapTupleIsValid(tup))
+				{
+					amprocForm = (Form_pg_amproc) GETSTRUCT(tup);
+
+					initStringInfo(&opfam);
+					getOpFamilyDescription(&opfam, amprocForm->amprocfamily, false);
+
+					/*------
+					   translator: %d is the function number, the first two
+					   %s's are data type names, the third %s is the description
+					   of the operator family, and the last %s is the textual
+					   form of the function with arguments.  */
+					appendStringInfo(&buffer, _("function %d (%s, %s) of %s: %s"),
+									 amprocForm->amprocnum,
+									 format_type_be(amprocForm->amproclefttype),
+									 format_type_be(amprocForm->amprocrighttype),
+									 opfam.data,
+									 format_procedure(amprocForm->amproc));
+
+					pfree(opfam.data);
+				}
+				else if (!missing_ok)
+				{
 					elog(ERROR, "could not find tuple for amproc entry %u",
 						 object->objectId);
-
-				amprocForm = (Form_pg_amproc) GETSTRUCT(tup);
-
-				initStringInfo(&opfam);
-				getOpFamilyDescription(&opfam, amprocForm->amprocfamily);
-
-				/*------
-				   translator: %d is the function number, the first two %s's
-				   are data type names, the third %s is the description of the
-				   operator family, and the last %s is the textual form of the
-				   function with arguments.  */
-				appendStringInfo(&buffer, _("function %d (%s, %s) of %s: %s"),
-								 amprocForm->amprocnum,
-								 format_type_be(amprocForm->amproclefttype),
-								 format_type_be(amprocForm->amprocrighttype),
-								 opfam.data,
-								 format_procedure(amprocForm->amproc));
-
-				pfree(opfam.data);
+				}
 
 				systable_endscan(amscan);
 				heap_close(amprocDesc, AccessShareLock);
@@ -3009,15 +3072,19 @@ getObjectDescription(const ObjectAddress *object)
 
 				tup = systable_getnext(rcscan);
 
-				if (!HeapTupleIsValid(tup))
+				if (HeapTupleIsValid(tup))
+				{
+					rule = (Form_pg_rewrite) GETSTRUCT(tup);
+
+					appendStringInfo(&buffer, _("rule %s on "),
+									 NameStr(rule->rulename));
+					getRelationDescription(&buffer, rule->ev_class, false);
+				}
+				else if (!missing_ok)
+				{
 					elog(ERROR, "could not find tuple for rule %u",
 						 object->objectId);
-
-				rule = (Form_pg_rewrite) GETSTRUCT(tup);
-
-				appendStringInfo(&buffer, _("rule %s on "),
-								 NameStr(rule->rulename));
-				getRelationDescription(&buffer, rule->ev_class);
+				}
 
 				systable_endscan(rcscan);
 				heap_close(ruleDesc, AccessShareLock);
@@ -3044,15 +3111,19 @@ getObjectDescription(const ObjectAddress *object)
 
 				tup = systable_getnext(tgscan);
 
-				if (!HeapTupleIsValid(tup))
+				if (HeapTupleIsValid(tup))
+				{
+					trig = (Form_pg_trigger) GETSTRUCT(tup);
+
+					appendStringInfo(&buffer, _("trigger %s on "),
+									 NameStr(trig->tgname));
+					getRelationDescription(&buffer, trig->tgrelid, false);
+				}
+				else if (!missing_ok)
+				{
 					elog(ERROR, "could not find tuple for trigger %u",
 						 object->objectId);
-
-				trig = (Form_pg_trigger) GETSTRUCT(tup);
-
-				appendStringInfo(&buffer, _("trigger %s on "),
-								 NameStr(trig->tgname));
-				getRelationDescription(&buffer, trig->tgrelid);
+				}
 
 				systable_endscan(tgscan);
 				heap_close(trigDesc, AccessShareLock);
@@ -3065,8 +3136,12 @@ getObjectDescription(const ObjectAddress *object)
 
 				nspname = get_namespace_name(object->objectId);
 				if (!nspname)
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for namespace %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("schema %s"), nspname);
 				break;
 			}
@@ -3079,8 +3154,12 @@ getObjectDescription(const ObjectAddress *object)
 				stxTup = SearchSysCache1(STATEXTOID,
 										 ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(stxTup))
-					elog(ERROR, "could not find tuple for statistics object %u",
+				{
+					if (missing_ok)
+						break;
+					elog(ERROR, "cache lookup failed for statistics object %u",
 						 object->objectId);
+				}
 
 				stxForm = (Form_pg_statistic_ext) GETSTRUCT(stxTup);
 
@@ -3098,8 +3177,12 @@ getObjectDescription(const ObjectAddress *object)
 				tup = SearchSysCache1(TSPARSEROID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for text search parser %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("text search parser %s"),
 								 NameStr(((Form_pg_ts_parser) GETSTRUCT(tup))->prsname));
 				ReleaseSysCache(tup);
@@ -3113,8 +3196,12 @@ getObjectDescription(const ObjectAddress *object)
 				tup = SearchSysCache1(TSDICTOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for text search dictionary %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("text search dictionary %s"),
 								 NameStr(((Form_pg_ts_dict) GETSTRUCT(tup))->dictname));
 				ReleaseSysCache(tup);
@@ -3128,8 +3215,12 @@ getObjectDescription(const ObjectAddress *object)
 				tup = SearchSysCache1(TSTEMPLATEOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for text search template %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("text search template %s"),
 								 NameStr(((Form_pg_ts_template) GETSTRUCT(tup))->tmplname));
 				ReleaseSysCache(tup);
@@ -3143,8 +3234,12 @@ getObjectDescription(const ObjectAddress *object)
 				tup = SearchSysCache1(TSCONFIGOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for text search configuration %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("text search configuration %s"),
 								 NameStr(((Form_pg_ts_config) GETSTRUCT(tup))->cfgname));
 				ReleaseSysCache(tup);
@@ -3153,8 +3248,11 @@ getObjectDescription(const ObjectAddress *object)
 
 		case OCLASS_ROLE:
 			{
-				appendStringInfo(&buffer, _("role %s"),
-								 GetUserNameFromId(object->objectId, false));
+				char	  *username = GetUserNameFromId(object->objectId,
+														missing_ok);
+
+				if (username)
+					appendStringInfo(&buffer, _("role %s"), username);
 				break;
 			}
 
@@ -3164,8 +3262,12 @@ getObjectDescription(const ObjectAddress *object)
 
 				datname = get_database_name(object->objectId);
 				if (!datname)
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for database %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("database %s"), datname);
 				break;
 			}
@@ -3176,8 +3278,12 @@ getObjectDescription(const ObjectAddress *object)
 
 				tblspace = get_tablespace_name(object->objectId);
 				if (!tblspace)
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for tablespace %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("tablespace %s"), tblspace);
 				break;
 			}
@@ -3186,8 +3292,9 @@ getObjectDescription(const ObjectAddress *object)
 			{
 				ForeignDataWrapper *fdw;
 
-				fdw = GetForeignDataWrapper(object->objectId, false);
-				appendStringInfo(&buffer, _("foreign-data wrapper %s"), fdw->fdwname);
+				fdw = GetForeignDataWrapper(object->objectId, missing_ok);
+				if (fdw)
+					appendStringInfo(&buffer, _("foreign-data wrapper %s"), fdw->fdwname);
 				break;
 			}
 
@@ -3195,8 +3302,9 @@ getObjectDescription(const ObjectAddress *object)
 			{
 				ForeignServer *srv;
 
-				srv = GetForeignServer(object->objectId, false);
-				appendStringInfo(&buffer, _("server %s"), srv->servername);
+				srv = GetForeignServer(object->objectId, missing_ok);
+				if (srv)
+					appendStringInfo(&buffer, _("server %s"), srv->servername);
 				break;
 			}
 
@@ -3211,8 +3319,13 @@ getObjectDescription(const ObjectAddress *object)
 				tup = SearchSysCache1(USERMAPPINGOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for user mapping %u",
 						 object->objectId);
+				}
+
 				umform = (Form_pg_user_mapping) GETSTRUCT(tup);
 				useid = umform->umuser;
 				srv = GetForeignServer(umform->umserver, false);
@@ -3250,8 +3363,16 @@ getObjectDescription(const ObjectAddress *object)
 				tup = systable_getnext(rcscan);
 
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+					{
+						systable_endscan(rcscan);
+						heap_close(defaclrel, AccessShareLock);
+						break;
+					}
 					elog(ERROR, "could not find tuple for default ACL %u",
 						 object->objectId);
+				}
 
 				defacl = (Form_pg_default_acl) GETSTRUCT(tup);
 
@@ -3308,8 +3429,12 @@ getObjectDescription(const ObjectAddress *object)
 
 				extname = get_extension_name(object->objectId);
 				if (!extname)
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for extension %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("extension %s"), extname);
 				break;
 			}
@@ -3321,8 +3446,12 @@ getObjectDescription(const ObjectAddress *object)
 				tup = SearchSysCache1(EVENTTRIGGEROID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for event trigger %u",
 						 object->objectId);
+				}
 				appendStringInfo(&buffer, _("event trigger %s"),
 								 NameStr(((Form_pg_event_trigger) GETSTRUCT(tup))->evtname));
 				ReleaseSysCache(tup);
@@ -3349,15 +3478,20 @@ getObjectDescription(const ObjectAddress *object)
 
 				tuple = systable_getnext(sscan);
 
-				if (!HeapTupleIsValid(tuple))
+				if (HeapTupleIsValid(tuple))
+				{
+					form_policy = (Form_pg_policy) GETSTRUCT(tuple);
+
+					appendStringInfo(&buffer, _("policy %s on "),
+									 NameStr(form_policy->polname));
+					getRelationDescription(&buffer, form_policy->polrelid,
+										   false);
+				}
+				else if (!missing_ok)
+				{
 					elog(ERROR, "could not find tuple for policy %u",
 						 object->objectId);
-
-				form_policy = (Form_pg_policy) GETSTRUCT(tuple);
-
-				appendStringInfo(&buffer, _("policy %s on "),
-								 NameStr(form_policy->polname));
-				getRelationDescription(&buffer, form_policy->polrelid);
+				}
 
 				systable_endscan(sscan);
 				heap_close(policy_rel, AccessShareLock);
@@ -3366,9 +3500,11 @@ getObjectDescription(const ObjectAddress *object)
 
 		case OCLASS_PUBLICATION:
 			{
-				appendStringInfo(&buffer, _("publication %s"),
-								 get_publication_name(object->objectId,
-													  false));
+				char *pubname = get_publication_name(object->objectId,
+													 missing_ok);
+
+				if (pubname)
+					appendStringInfo(&buffer, _("publication %s"), pubname);
 				break;
 			}
 
@@ -3381,8 +3517,12 @@ getObjectDescription(const ObjectAddress *object)
 				tup = SearchSysCache1(PUBLICATIONREL,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for publication table %u",
 						 object->objectId);
+				}
 
 				prform = (Form_pg_publication_rel) GETSTRUCT(tup);
 				pubname = get_publication_name(prform->prpubid, false);
@@ -3395,8 +3535,10 @@ getObjectDescription(const ObjectAddress *object)
 
 		case OCLASS_SUBSCRIPTION:
 			{
-				appendStringInfo(&buffer, _("subscription %s"),
-								 get_subscription_name(object->objectId, false));
+				char *subname = get_subscription_name(object->objectId, missing_ok);
+
+				if (subname)
+					appendStringInfo(&buffer, _("subscription %s"), subname);
 				break;
 			}
 
@@ -3408,8 +3550,12 @@ getObjectDescription(const ObjectAddress *object)
 				trfTup = SearchSysCache1(TRFOID,
 										 ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(trfTup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "could not find tuple for transform %u",
 						 object->objectId);
+				}
 
 				trfForm = (Form_pg_transform) GETSTRUCT(trfTup);
 
@@ -3427,6 +3573,10 @@ getObjectDescription(const ObjectAddress *object)
 			 */
 	}
 
+	/* an empty buffer is equivalent to no object found */
+	if (buffer.len == 0)
+		return NULL;
+
 	return buffer.data;
 }
 
@@ -3442,14 +3592,14 @@ getObjectDescriptionOids(Oid classid, Oid objid)
 	address.objectId = objid;
 	address.objectSubId = 0;
 
-	return getObjectDescription(&address);
+	return getObjectDescription(&address, false);
 }
 
 /*
  * subroutine for getObjectDescription: describe a relation
  */
 static void
-getRelationDescription(StringInfo buffer, Oid relid)
+getRelationDescription(StringInfo buffer, Oid relid, bool missing_ok)
 {
 	HeapTuple	relTup;
 	Form_pg_class relForm;
@@ -3459,7 +3609,11 @@ getRelationDescription(StringInfo buffer, Oid relid)
 	relTup = SearchSysCache1(RELOID,
 							 ObjectIdGetDatum(relid));
 	if (!HeapTupleIsValid(relTup))
+	{
+		if (missing_ok)
+			return;
 		elog(ERROR, "cache lookup failed for relation %u", relid);
+	}
 	relForm = (Form_pg_class) GETSTRUCT(relTup);
 
 	/* Qualify the name if not visible in search path */
@@ -3519,7 +3673,7 @@ getRelationDescription(StringInfo buffer, Oid relid)
  * subroutine for getObjectDescription: describe an operator family
  */
 static void
-getOpFamilyDescription(StringInfo buffer, Oid opfid)
+getOpFamilyDescription(StringInfo buffer, Oid opfid, bool missing_ok)
 {
 	HeapTuple	opfTup;
 	Form_pg_opfamily opfForm;
@@ -3529,7 +3683,11 @@ getOpFamilyDescription(StringInfo buffer, Oid opfid)
 
 	opfTup = SearchSysCache1(OPFAMILYOID, ObjectIdGetDatum(opfid));
 	if (!HeapTupleIsValid(opfTup))
+	{
+		if (missing_ok)
+			return;
 		elog(ERROR, "cache lookup failed for opfamily %u", opfid);
+	}
 	opfForm = (Form_pg_opfamily) GETSTRUCT(opfTup);
 
 	amTup = SearchSysCache1(AMOID, ObjectIdGetDatum(opfForm->opfmethod));
@@ -3573,7 +3731,11 @@ pg_describe_object(PG_FUNCTION_ARGS)
 	address.objectId = objid;
 	address.objectSubId = objsubid;
 
-	description = getObjectDescription(&address);
+	description = getObjectDescription(&address, true);
+
+	if (description == NULL)
+		PG_RETURN_NULL();
+
 	PG_RETURN_TEXT_P(cstring_to_text(description));
 }
 
@@ -3588,6 +3750,8 @@ pg_identify_object(PG_FUNCTION_ARGS)
 	int32		objsubid = PG_GETARG_INT32(2);
 	Oid			schema_oid = InvalidOid;
 	const char *objname = NULL;
+	char	   *objtype;
+	char	   *objidentity;
 	ObjectAddress address;
 	Datum		values[4];
 	bool		nulls[4];
@@ -3661,8 +3825,14 @@ pg_identify_object(PG_FUNCTION_ARGS)
 	}
 
 	/* object type */
-	values[0] = CStringGetTextDatum(getObjectTypeDescription(&address));
-	nulls[0] = false;
+	objtype = getObjectTypeDescription(&address, true);
+	if (objtype)
+	{
+		values[0] = CStringGetTextDatum(getObjectTypeDescription(&address, false));
+		nulls[0] = false;
+	}
+	else
+		nulls[0] = true;
 
 	/* schema name */
 	if (OidIsValid(schema_oid))
@@ -3685,8 +3855,14 @@ pg_identify_object(PG_FUNCTION_ARGS)
 		nulls[2] = true;
 
 	/* object identity */
-	values[3] = CStringGetTextDatum(getObjectIdentity(&address));
-	nulls[3] = false;
+	objidentity = getObjectIdentity(&address, true);
+	if (objidentity)
+	{
+		values[3] = CStringGetTextDatum(getObjectIdentity(&address, true));
+		nulls[3] = false;
+	}
+	else
+		nulls[3] = true;
 
 	htup = heap_form_tuple(tupdesc, values, nulls);
 
@@ -3703,6 +3879,7 @@ pg_identify_object_as_address(PG_FUNCTION_ARGS)
 	Oid			objid = PG_GETARG_OID(1);
 	int32		objsubid = PG_GETARG_INT32(2);
 	ObjectAddress address;
+	char	   *type;
 	char	   *identity;
 	List	   *names;
 	List	   *args;
@@ -3730,23 +3907,37 @@ pg_identify_object_as_address(PG_FUNCTION_ARGS)
 	tupdesc = BlessTupleDesc(tupdesc);
 
 	/* object type */
-	values[0] = CStringGetTextDatum(getObjectTypeDescription(&address));
-	nulls[0] = false;
+	type = getObjectTypeDescription(&address, true);
+	if (type)
+	{
+		values[0] = CStringGetTextDatum(getObjectTypeDescription(&address, false));
+		nulls[0] = false;
+	}
+	else
+		nulls[0] = true;
 
 	/* object identity */
-	identity = getObjectIdentityParts(&address, &names, &args);
-	pfree(identity);
+	identity = getObjectIdentityParts(&address, &names, &args, true);
+	if (identity == NULL)
+	{
+		nulls[1] = true;
+		nulls[2] = true;
+	}
+	else
+	{
+		pfree(identity);
 
-	/* object_names */
-	values[1] = PointerGetDatum(strlist_to_textarray(names));
-	nulls[1] = false;
+		/* object_names */
+		values[1] = PointerGetDatum(strlist_to_textarray(names));
+		nulls[1] = false;
 
-	/* object_args */
-	if (args)
-		values[2] = PointerGetDatum(strlist_to_textarray(args));
-	else
-		values[2] = PointerGetDatum(construct_empty_array(TEXTOID));
-	nulls[2] = false;
+		/* object_args */
+		if (args)
+			values[2] = PointerGetDatum(strlist_to_textarray(args));
+		else
+			values[2] = PointerGetDatum(construct_empty_array(TEXTOID));
+		nulls[2] = false;
+	}
 
 	htup = heap_form_tuple(tupdesc, values, nulls);
 
@@ -3760,7 +3951,7 @@ pg_identify_object_as_address(PG_FUNCTION_ARGS)
  * Keep ObjectTypeMap in sync with this.
  */
 char *
-getObjectTypeDescription(const ObjectAddress *object)
+getObjectTypeDescription(const ObjectAddress *object, bool missing_ok)
 {
 	StringInfoData buffer;
 
@@ -3770,11 +3961,13 @@ getObjectTypeDescription(const ObjectAddress *object)
 	{
 		case OCLASS_CLASS:
 			getRelationTypeDescription(&buffer, object->objectId,
-									   object->objectSubId);
+									   object->objectSubId,
+									   missing_ok);
 			break;
 
 		case OCLASS_PROC:
-			getProcedureTypeDescription(&buffer, object->objectId);
+			getProcedureTypeDescription(&buffer, object->objectId,
+										missing_ok);
 			break;
 
 		case OCLASS_TYPE:
@@ -3790,7 +3983,8 @@ getObjectTypeDescription(const ObjectAddress *object)
 			break;
 
 		case OCLASS_CONSTRAINT:
-			getConstraintTypeDescription(&buffer, object->objectId);
+			getConstraintTypeDescription(&buffer, object->objectId,
+										 missing_ok);
 			break;
 
 		case OCLASS_CONVERSION:
@@ -3927,6 +4121,10 @@ getObjectTypeDescription(const ObjectAddress *object)
 			 */
 	}
 
+	/* an empty string is equivalent to no object found */
+	if (buffer.len == 0)
+		return NULL;
+
 	return buffer.data;
 }
 
@@ -3934,7 +4132,8 @@ getObjectTypeDescription(const ObjectAddress *object)
  * subroutine for getObjectTypeDescription: describe a relation type
  */
 static void
-getRelationTypeDescription(StringInfo buffer, Oid relid, int32 objectSubId)
+getRelationTypeDescription(StringInfo buffer, Oid relid, int32 objectSubId,
+						   bool missing_ok)
 {
 	HeapTuple	relTup;
 	Form_pg_class relForm;
@@ -3942,7 +4141,14 @@ getRelationTypeDescription(StringInfo buffer, Oid relid, int32 objectSubId)
 	relTup = SearchSysCache1(RELOID,
 							 ObjectIdGetDatum(relid));
 	if (!HeapTupleIsValid(relTup))
+	{
+		if (missing_ok)
+		{
+			Assert(buffer->len == 0);
+			return;
+		}
 		elog(ERROR, "cache lookup failed for relation %u", relid);
+	}
 	relForm = (Form_pg_class) GETSTRUCT(relTup);
 
 	switch (relForm->relkind)
@@ -3988,7 +4194,7 @@ getRelationTypeDescription(StringInfo buffer, Oid relid, int32 objectSubId)
  * subroutine for getObjectTypeDescription: describe a constraint type
  */
 static void
-getConstraintTypeDescription(StringInfo buffer, Oid constroid)
+getConstraintTypeDescription(StringInfo buffer, Oid constroid, bool missing_ok)
 {
 	Relation	constrRel;
 	HeapTuple	constrTup;
@@ -3997,7 +4203,14 @@ getConstraintTypeDescription(StringInfo buffer, Oid constroid)
 	constrRel = heap_open(ConstraintRelationId, AccessShareLock);
 	constrTup = get_catalog_object_by_oid(constrRel, constroid);
 	if (!HeapTupleIsValid(constrTup))
+	{
+		if (missing_ok)
+		{
+			heap_close(constrRel, AccessShareLock);
+			return;
+		}
 		elog(ERROR, "cache lookup failed for constraint %u", constroid);
+	}
 
 	constrForm = (Form_pg_constraint) GETSTRUCT(constrTup);
 
@@ -4015,7 +4228,8 @@ getConstraintTypeDescription(StringInfo buffer, Oid constroid)
  * subroutine for getObjectTypeDescription: describe a procedure type
  */
 static void
-getProcedureTypeDescription(StringInfo buffer, Oid procid)
+getProcedureTypeDescription(StringInfo buffer, Oid procid,
+							bool missing_ok)
 {
 	HeapTuple	procTup;
 	Form_pg_proc procForm;
@@ -4023,7 +4237,11 @@ getProcedureTypeDescription(StringInfo buffer, Oid procid)
 	procTup = SearchSysCache1(PROCOID,
 							  ObjectIdGetDatum(procid));
 	if (!HeapTupleIsValid(procTup))
+	{
+		if (missing_ok)
+			return;
 		elog(ERROR, "cache lookup failed for procedure %u", procid);
+	}
 	procForm = (Form_pg_proc) GETSTRUCT(procTup);
 
 	if (procForm->proisagg)
@@ -4038,12 +4256,13 @@ getProcedureTypeDescription(StringInfo buffer, Oid procid)
  * Obtain a given object's identity, as a palloc'ed string.
  *
  * This is for machine consumption, so it's not translated.  All elements are
- * schema-qualified when appropriate.
+ * schema-qualified when appropriate. Returns NULL if the object defined
+ * cannot be found.
  */
 char *
-getObjectIdentity(const ObjectAddress *object)
+getObjectIdentity(const ObjectAddress *object, bool missing_ok)
 {
-	return getObjectIdentityParts(object, NULL, NULL);
+	return getObjectIdentityParts(object, NULL, NULL, missing_ok);
 }
 
 /*
@@ -4052,11 +4271,13 @@ getObjectIdentity(const ObjectAddress *object)
  * There are two sets of return values: the identity itself as a palloc'd
  * string is returned.  objname and objargs, if not NULL, are output parameters
  * that receive lists of C-strings that are useful to give back to
- * get_object_address() to reconstruct the ObjectAddress.
+ * get_object_address() to reconstruct the ObjectAddress. Returns NULL if
+ * the object defined cannot be found.
  */
 char *
 getObjectIdentityParts(const ObjectAddress *object,
-					   List **objname, List **objargs)
+					   List **objname, List **objargs,
+					   bool missing_ok)
 {
 	StringInfoData buffer;
 
@@ -4078,32 +4299,41 @@ getObjectIdentityParts(const ObjectAddress *object,
 	switch (getObjectClass(object))
 	{
 		case OCLASS_CLASS:
-			getRelationIdentity(&buffer, object->objectId, objname);
+			getRelationIdentity(&buffer, object->objectId, objname,
+								missing_ok);
+			if (objname && *objname == NIL)
+				break;
 			if (object->objectSubId != 0)
 			{
-				char	   *attr;
-
-				attr = get_attname(object->objectId,
-								   object->objectSubId,
-								   false);
-				appendStringInfo(&buffer, ".%s", quote_identifier(attr));
-				if (objname)
-					*objname = lappend(*objname, attr);
+				char	   *attr = get_attname(object->objectId,
+											   object->objectSubId,
+											   missing_ok);
+				if (attr)
+				{
+					appendStringInfo(&buffer, ".%s", quote_identifier(attr));
+					if (objname)
+						*objname = lappend(*objname, attr);
+				}
 			}
 			break;
 
 		case OCLASS_PROC:
+			/* returns numerical OID for undefined function */
 			appendStringInfoString(&buffer,
 								   format_procedure_qualified(object->objectId));
 			if (objname)
-				format_procedure_parts(object->objectId, objname, objargs);
+				format_procedure_parts(object->objectId, objname, objargs,
+									   missing_ok);
 			break;
 
 		case OCLASS_TYPE:
 			{
 				char	   *typeout;
 
-				typeout = format_type_be_qualified(object->objectId);
+				/* returns "???" for undefined type */
+				typeout = format_type_extended(object->objectId, -1, false,
+											   missing_ok, true);
+
 				appendStringInfoString(&buffer, typeout);
 				if (objname)
 					*objname = list_make1(typeout);
@@ -4121,8 +4351,15 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = get_catalog_object_by_oid(castRel, object->objectId);
 
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+					{
+						heap_close(castRel, AccessShareLock);
+						break;
+					}
 					elog(ERROR, "could not find tuple for cast %u",
 						 object->objectId);
+				}
 
 				castForm = (Form_pg_cast) GETSTRUCT(tup);
 
@@ -4149,8 +4386,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				collTup = SearchSysCache1(COLLOID,
 										  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(collTup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for collation %u",
 						 object->objectId);
+				}
 				coll = (Form_pg_collation) GETSTRUCT(collTup);
 				schema = get_namespace_name_or_temp(coll->collnamespace);
 				appendStringInfoString(&buffer,
@@ -4171,15 +4412,20 @@ getObjectIdentityParts(const ObjectAddress *object,
 				conTup = SearchSysCache1(CONSTROID,
 										 ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(conTup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for constraint %u",
 						 object->objectId);
+				}
 				con = (Form_pg_constraint) GETSTRUCT(conTup);
 
 				if (OidIsValid(con->conrelid))
 				{
 					appendStringInfo(&buffer, "%s on ",
 									 quote_identifier(NameStr(con->conname)));
-					getRelationIdentity(&buffer, con->conrelid, objname);
+					getRelationIdentity(&buffer, con->conrelid, objname,
+										false);
 					if (objname)
 						*objname = lappend(*objname, pstrdup(NameStr(con->conname)));
 				}
@@ -4194,7 +4440,8 @@ getObjectIdentityParts(const ObjectAddress *object,
 
 					appendStringInfo(&buffer, "%s on %s",
 									 quote_identifier(NameStr(con->conname)),
-									 getObjectIdentityParts(&domain, objname, objargs));
+									 getObjectIdentityParts(&domain, objname,
+															objargs, false));
 
 					if (objname)
 						*objargs = lappend(*objargs, pstrdup(NameStr(con->conname)));
@@ -4213,8 +4460,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				conTup = SearchSysCache1(CONVOID,
 										 ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(conTup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for conversion %u",
 						 object->objectId);
+				}
 				conForm = (Form_pg_conversion) GETSTRUCT(conTup);
 				schema = get_namespace_name_or_temp(conForm->connamespace);
 				appendStringInfoString(&buffer,
@@ -4249,19 +4500,24 @@ getObjectIdentityParts(const ObjectAddress *object,
 
 				tup = systable_getnext(adscan);
 
-				if (!HeapTupleIsValid(tup))
-					elog(ERROR, "could not find tuple for attrdef %u",
-						 object->objectId);
-
-				attrdef = (Form_pg_attrdef) GETSTRUCT(tup);
+				if (HeapTupleIsValid(tup))
+				{
+					attrdef = (Form_pg_attrdef) GETSTRUCT(tup);
 
-				colobject.classId = RelationRelationId;
-				colobject.objectId = attrdef->adrelid;
-				colobject.objectSubId = attrdef->adnum;
+					colobject.classId = RelationRelationId;
+					colobject.objectId = attrdef->adrelid;
+					colobject.objectSubId = attrdef->adnum;
 
-				appendStringInfo(&buffer, "for %s",
-								 getObjectIdentityParts(&colobject,
-														objname, objargs));
+					appendStringInfo(&buffer, "for %s",
+									 getObjectIdentityParts(&colobject,
+															objname, objargs,
+															false));
+				}
+				else if (!missing_ok)
+				{
+					elog(ERROR, "could not find tuple for attrdef %u",
+						 object->objectId);
+				}
 
 				systable_endscan(adscan);
 				heap_close(attrdefDesc, AccessShareLock);
@@ -4276,8 +4532,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				langTup = SearchSysCache1(LANGOID,
 										  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(langTup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for language %u",
 						 object->objectId);
+				}
 				langForm = (Form_pg_language) GETSTRUCT(langTup);
 				appendStringInfoString(&buffer,
 									   quote_identifier(NameStr(langForm->lanname)));
@@ -4294,10 +4554,11 @@ getObjectIdentityParts(const ObjectAddress *object,
 			break;
 
 		case OCLASS_OPERATOR:
+			/* returns numerical OID for undefined operator */
 			appendStringInfoString(&buffer,
 								   format_operator_qualified(object->objectId));
 			if (objname)
-				format_operator_parts(object->objectId, objname, objargs);
+				format_operator_parts(object->objectId, objname, objargs, missing_ok);
 			break;
 
 		case OCLASS_OPCLASS:
@@ -4311,8 +4572,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				opcTup = SearchSysCache1(CLAOID,
 										 ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(opcTup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for opclass %u",
 						 object->objectId);
+				}
 				opcForm = (Form_pg_opclass) GETSTRUCT(opcTup);
 				schema = get_namespace_name_or_temp(opcForm->opcnamespace);
 
@@ -4338,7 +4603,8 @@ getObjectIdentityParts(const ObjectAddress *object,
 			}
 
 		case OCLASS_OPFAMILY:
-			getOpFamilyIdentity(&buffer, object->objectId, objname);
+			getOpFamilyIdentity(&buffer, object->objectId, objname,
+								missing_ok);
 			break;
 
 		case OCLASS_AM:
@@ -4347,8 +4613,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 
 				amname = get_am_name(object->objectId);
 				if (!amname)
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for access method %u",
 						 object->objectId);
+				}
 				appendStringInfoString(&buffer, quote_identifier(amname));
 				if (objname)
 					*objname = list_make1(amname);
@@ -4380,13 +4650,22 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = systable_getnext(amscan);
 
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+					{
+						systable_endscan(amscan);
+						heap_close(amopDesc, AccessShareLock);
+						break;
+					}
 					elog(ERROR, "could not find tuple for amop entry %u",
 						 object->objectId);
+				}
 
 				amopForm = (Form_pg_amop) GETSTRUCT(tup);
 
 				initStringInfo(&opfam);
-				getOpFamilyIdentity(&opfam, amopForm->amopfamily, objname);
+				getOpFamilyIdentity(&opfam, amopForm->amopfamily, objname,
+									false);
 
 				ltype = format_type_be_qualified(amopForm->amoplefttype);
 				rtype = format_type_be_qualified(amopForm->amoprighttype);
@@ -4434,13 +4713,22 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = systable_getnext(amscan);
 
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+					{
+						systable_endscan(amscan);
+						heap_close(amprocDesc, AccessShareLock);
+						break;
+					}
 					elog(ERROR, "could not find tuple for amproc entry %u",
 						 object->objectId);
+				}
 
 				amprocForm = (Form_pg_amproc) GETSTRUCT(tup);
 
 				initStringInfo(&opfam);
-				getOpFamilyIdentity(&opfam, amprocForm->amprocfamily, objname);
+				getOpFamilyIdentity(&opfam, amprocForm->amprocfamily, objname,
+									false);
 
 				ltype = format_type_be_qualified(amprocForm->amproclefttype);
 				rtype = format_type_be_qualified(amprocForm->amprocrighttype);
@@ -4474,14 +4762,21 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = get_catalog_object_by_oid(ruleDesc, object->objectId);
 
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+					{
+						heap_close(ruleDesc, AccessShareLock);
+						break;
+					}
 					elog(ERROR, "could not find tuple for rule %u",
 						 object->objectId);
+				}
 
 				rule = (Form_pg_rewrite) GETSTRUCT(tup);
 
 				appendStringInfo(&buffer, "%s on ",
 								 quote_identifier(NameStr(rule->rulename)));
-				getRelationIdentity(&buffer, rule->ev_class, objname);
+				getRelationIdentity(&buffer, rule->ev_class, objname, false);
 				if (objname)
 					*objname = lappend(*objname, pstrdup(NameStr(rule->rulename)));
 
@@ -4500,14 +4795,21 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = get_catalog_object_by_oid(trigDesc, object->objectId);
 
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+					{
+						heap_close(trigDesc, AccessShareLock);
+						break;
+					}
 					elog(ERROR, "could not find tuple for trigger %u",
 						 object->objectId);
+				}
 
 				trig = (Form_pg_trigger) GETSTRUCT(tup);
 
 				appendStringInfo(&buffer, "%s on ",
 								 quote_identifier(NameStr(trig->tgname)));
-				getRelationIdentity(&buffer, trig->tgrelid, objname);
+				getRelationIdentity(&buffer, trig->tgrelid, objname, false);
 				if (objname)
 					*objname = lappend(*objname, pstrdup(NameStr(trig->tgname)));
 
@@ -4521,8 +4823,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 
 				nspname = get_namespace_name_or_temp(object->objectId);
 				if (!nspname)
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for namespace %u",
 						 object->objectId);
+				}
 				appendStringInfoString(&buffer,
 									   quote_identifier(nspname));
 				if (objname)
@@ -4539,8 +4845,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = SearchSysCache1(STATEXTOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for statistics object %u",
 						 object->objectId);
+				}
 				formStatistic = (Form_pg_statistic_ext) GETSTRUCT(tup);
 				schema = get_namespace_name_or_temp(formStatistic->stxnamespace);
 				appendStringInfoString(&buffer,
@@ -4562,8 +4872,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = SearchSysCache1(TSPARSEROID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for text search parser %u",
 						 object->objectId);
+				}
 				formParser = (Form_pg_ts_parser) GETSTRUCT(tup);
 				schema = get_namespace_name_or_temp(formParser->prsnamespace);
 				appendStringInfoString(&buffer,
@@ -4585,8 +4899,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = SearchSysCache1(TSDICTOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for text search dictionary %u",
 						 object->objectId);
+				}
 				formDict = (Form_pg_ts_dict) GETSTRUCT(tup);
 				schema = get_namespace_name_or_temp(formDict->dictnamespace);
 				appendStringInfoString(&buffer,
@@ -4608,8 +4926,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = SearchSysCache1(TSTEMPLATEOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for text search template %u",
 						 object->objectId);
+				}
 				formTmpl = (Form_pg_ts_template) GETSTRUCT(tup);
 				schema = get_namespace_name_or_temp(formTmpl->tmplnamespace);
 				appendStringInfoString(&buffer,
@@ -4631,8 +4953,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = SearchSysCache1(TSCONFIGOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for text search configuration %u",
 						 object->objectId);
+				}
 				formCfg = (Form_pg_ts_config) GETSTRUCT(tup);
 				schema = get_namespace_name_or_temp(formCfg->cfgnamespace);
 				appendStringInfoString(&buffer,
@@ -4649,7 +4975,9 @@ getObjectIdentityParts(const ObjectAddress *object,
 			{
 				char	   *username;
 
-				username = GetUserNameFromId(object->objectId, false);
+				username = GetUserNameFromId(object->objectId, missing_ok);
+				if (!username)
+					break;
 				if (objname)
 					*objname = list_make1(username);
 				appendStringInfoString(&buffer,
@@ -4663,8 +4991,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 
 				datname = get_database_name(object->objectId);
 				if (!datname)
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for database %u",
 						 object->objectId);
+				}
 				if (objname)
 					*objname = list_make1(datname);
 				appendStringInfoString(&buffer,
@@ -4678,8 +5010,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 
 				tblspace = get_tablespace_name(object->objectId);
 				if (!tblspace)
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for tablespace %u",
 						 object->objectId);
+				}
 				if (objname)
 					*objname = list_make1(tblspace);
 				appendStringInfoString(&buffer,
@@ -4691,10 +5027,13 @@ getObjectIdentityParts(const ObjectAddress *object,
 			{
 				ForeignDataWrapper *fdw;
 
-				fdw = GetForeignDataWrapper(object->objectId, false);
-				appendStringInfoString(&buffer, quote_identifier(fdw->fdwname));
-				if (objname)
-					*objname = list_make1(pstrdup(fdw->fdwname));
+				fdw = GetForeignDataWrapper(object->objectId, missing_ok);
+				if (fdw)
+				{
+					appendStringInfoString(&buffer, quote_identifier(fdw->fdwname));
+					if (objname)
+						*objname = list_make1(pstrdup(fdw->fdwname));
+				}
 				break;
 			}
 
@@ -4702,11 +5041,14 @@ getObjectIdentityParts(const ObjectAddress *object,
 			{
 				ForeignServer *srv;
 
-				srv = GetForeignServer(object->objectId, false);
-				appendStringInfoString(&buffer,
-									   quote_identifier(srv->servername));
-				if (objname)
-					*objname = list_make1(pstrdup(srv->servername));
+				srv = GetForeignServer(object->objectId, missing_ok);
+				if (srv)
+				{
+					appendStringInfoString(&buffer,
+										   quote_identifier(srv->servername));
+					if (objname)
+						*objname = list_make1(pstrdup(srv->servername));
+				}
 				break;
 			}
 
@@ -4721,8 +5063,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = SearchSysCache1(USERMAPPINGOID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for user mapping %u",
 						 object->objectId);
+				}
 				umform = (Form_pg_user_mapping) GETSTRUCT(tup);
 				useid = umform->umuser;
 				srv = GetForeignServer(umform->umserver, false);
@@ -4769,8 +5115,16 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = systable_getnext(rcscan);
 
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+					{
+						systable_endscan(rcscan);
+						heap_close(defaclrel, AccessShareLock);
+						break;
+					}
 					elog(ERROR, "could not find tuple for default ACL %u",
 						 object->objectId);
+				}
 
 				defacl = (Form_pg_default_acl) GETSTRUCT(tup);
 
@@ -4832,8 +5186,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 
 				extname = get_extension_name(object->objectId);
 				if (!extname)
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for extension %u",
 						 object->objectId);
+				}
 				appendStringInfoString(&buffer, quote_identifier(extname));
 				if (objname)
 					*objname = list_make1(extname);
@@ -4852,8 +5210,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = SearchSysCache1(EVENTTRIGGEROID,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for event trigger %u",
 						 object->objectId);
+				}
 				trigForm = (Form_pg_event_trigger) GETSTRUCT(tup);
 				appendStringInfoString(&buffer,
 									   quote_identifier(NameStr(trigForm->evtname)));
@@ -4872,14 +5234,21 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = get_catalog_object_by_oid(polDesc, object->objectId);
 
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+					{
+						heap_close(polDesc, AccessShareLock);
+						break;
+					}
 					elog(ERROR, "could not find tuple for policy %u",
 						 object->objectId);
+				}
 
 				policy = (Form_pg_policy) GETSTRUCT(tup);
 
 				appendStringInfo(&buffer, "%s on ",
 								 quote_identifier(NameStr(policy->polname)));
-				getRelationIdentity(&buffer, policy->polrelid, objname);
+				getRelationIdentity(&buffer, policy->polrelid, objname, false);
 				if (objname)
 					*objname = lappend(*objname, pstrdup(NameStr(policy->polname)));
 
@@ -4891,11 +5260,14 @@ getObjectIdentityParts(const ObjectAddress *object,
 			{
 				char	   *pubname;
 
-				pubname = get_publication_name(object->objectId, false);
-				appendStringInfoString(&buffer,
-									   quote_identifier(pubname));
-				if (objname)
-					*objname = list_make1(pubname);
+				pubname = get_publication_name(object->objectId, missing_ok);
+				if (pubname)
+				{
+					appendStringInfoString(&buffer,
+										   quote_identifier(pubname));
+					if (objname)
+						*objname = list_make1(pubname);
+				}
 				break;
 			}
 
@@ -4908,8 +5280,12 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = SearchSysCache1(PUBLICATIONREL,
 									  ObjectIdGetDatum(object->objectId));
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+						break;
 					elog(ERROR, "cache lookup failed for publication table %u",
 						 object->objectId);
+				}
 
 				prform = (Form_pg_publication_rel) GETSTRUCT(tup);
 				pubname = get_publication_name(prform->prpubid, false);
@@ -4919,7 +5295,8 @@ getObjectIdentityParts(const ObjectAddress *object,
 
 				if (objname)
 				{
-					getRelationIdentity(&buffer, prform->prrelid, objname);
+					getRelationIdentity(&buffer, prform->prrelid, objname,
+										false);
 					*objargs = list_make1(pubname);
 				}
 
@@ -4931,11 +5308,14 @@ getObjectIdentityParts(const ObjectAddress *object,
 			{
 				char	   *subname;
 
-				subname = get_subscription_name(object->objectId, false);
-				appendStringInfoString(&buffer,
-									   quote_identifier(subname));
-				if (objname)
-					*objname = list_make1(subname);
+				subname = get_subscription_name(object->objectId, missing_ok);
+				if (subname)
+				{
+					appendStringInfoString(&buffer,
+										   quote_identifier(subname));
+					if (objname)
+						*objname = list_make1(subname);
+				}
 				break;
 			}
 
@@ -4952,8 +5332,15 @@ getObjectIdentityParts(const ObjectAddress *object,
 				tup = get_catalog_object_by_oid(transformDesc, object->objectId);
 
 				if (!HeapTupleIsValid(tup))
+				{
+					if (missing_ok)
+					{
+						heap_close(transformDesc, AccessShareLock);
+						break;
+					}
 					elog(ERROR, "could not find tuple for transform %u",
 						 object->objectId);
+				}
 
 				transform = (Form_pg_transform) GETSTRUCT(tup);
 
@@ -4979,20 +5366,20 @@ getObjectIdentityParts(const ObjectAddress *object,
 			 */
 	}
 
-	/*
-	 * If a get_object_address representation was requested, make sure we are
-	 * providing one.  We don't check objargs, because many of the cases above
-	 * leave it as NIL.
-	 */
-	if (objname && *objname == NIL)
-		elog(ERROR, "requested object address for unsupported object class %d: text result \"%s\"",
-			 (int) getObjectClass(object), buffer.data);
+	/* an empty string is equivalent to no object defined */
+	if (buffer.len == 0)
+	{
+		Assert((objname == NULL || *objname == NIL) &&
+			   (objargs == NULL || *objargs == NIL));
+		return NULL;
+	}
 
 	return buffer.data;
 }
 
 static void
-getOpFamilyIdentity(StringInfo buffer, Oid opfid, List **object)
+getOpFamilyIdentity(StringInfo buffer, Oid opfid, List **object,
+					bool missing_ok)
 {
 	HeapTuple	opfTup;
 	Form_pg_opfamily opfForm;
@@ -5002,7 +5389,11 @@ getOpFamilyIdentity(StringInfo buffer, Oid opfid, List **object)
 
 	opfTup = SearchSysCache1(OPFAMILYOID, ObjectIdGetDatum(opfid));
 	if (!HeapTupleIsValid(opfTup))
+	{
+		if (missing_ok)
+			return;
 		elog(ERROR, "cache lookup failed for opfamily %u", opfid);
+	}
 	opfForm = (Form_pg_opfamily) GETSTRUCT(opfTup);
 
 	amTup = SearchSysCache1(AMOID, ObjectIdGetDatum(opfForm->opfmethod));
@@ -5031,7 +5422,8 @@ getOpFamilyIdentity(StringInfo buffer, Oid opfid, List **object)
  * StringInfo.
  */
 static void
-getRelationIdentity(StringInfo buffer, Oid relid, List **object)
+getRelationIdentity(StringInfo buffer, Oid relid, List **object,
+					bool missing_ok)
 {
 	HeapTuple	relTup;
 	Form_pg_class relForm;
@@ -5040,7 +5432,15 @@ getRelationIdentity(StringInfo buffer, Oid relid, List **object)
 	relTup = SearchSysCache1(RELOID,
 							 ObjectIdGetDatum(relid));
 	if (!HeapTupleIsValid(relTup))
+	{
+		if (missing_ok)
+		{
+			if (object)
+				*object = NIL;
+			return;
+		}
 		elog(ERROR, "cache lookup failed for relation %u", relid);
+	}
 	relForm = (Form_pg_class) GETSTRUCT(relTup);
 
 	schema = get_namespace_name_or_temp(relForm->relnamespace);
diff --git a/src/backend/catalog/pg_depend.c b/src/backend/catalog/pg_depend.c
index cf0086b9bd..56d429d8dc 100644
--- a/src/backend/catalog/pg_depend.c
+++ b/src/backend/catalog/pg_depend.c
@@ -161,7 +161,7 @@ recordDependencyOnCurrentExtension(const ObjectAddress *object,
 				ereport(ERROR,
 						(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
 						 errmsg("%s is already a member of extension \"%s\"",
-								getObjectDescription(object),
+								getObjectDescription(object, false),
 								get_extension_name(oldext))));
 			}
 		}
@@ -320,7 +320,7 @@ changeDependencyFor(Oid classId, Oid objectId,
 		ereport(ERROR,
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 				 errmsg("cannot remove dependency on %s because it is a system object",
-						getObjectDescription(&objAddr))));
+						getObjectDescription(&objAddr, false))));
 
 	/*
 	 * We can handle adding a dependency on something pinned, though, since
diff --git a/src/backend/catalog/pg_shdepend.c b/src/backend/catalog/pg_shdepend.c
index 31b09a1da5..dbd3f2826c 100644
--- a/src/backend/catalog/pg_shdepend.c
+++ b/src/backend/catalog/pg_shdepend.c
@@ -572,7 +572,7 @@ checkSharedDependencies(Oid classId, Oid objectId,
 			ereport(ERROR,
 					(errcode(ERRCODE_DEPENDENT_OBJECTS_STILL_EXIST),
 					 errmsg("cannot drop %s because it is required by the database system",
-							getObjectDescription(&object))));
+							getObjectDescription(&object, false))));
 		}
 
 		object.classId = sdepForm->classid;
@@ -1063,7 +1063,7 @@ storeObjectDescription(StringInfo descs,
 					   SharedDependencyType deptype,
 					   int count)
 {
-	char	   *objdesc = getObjectDescription(object);
+	char	   *objdesc = getObjectDescription(object, false);
 
 	/* separate entries with a newline */
 	if (descs->len != 0)
@@ -1199,7 +1199,7 @@ shdepDropOwned(List *roleids, DropBehavior behavior)
 					(errcode(ERRCODE_DEPENDENT_OBJECTS_STILL_EXIST),
 					 errmsg("cannot drop objects owned by %s because they are "
 							"required by the database system",
-							getObjectDescription(&obj))));
+							getObjectDescription(&obj, false))));
 		}
 
 		ScanKeyInit(&key[0],
@@ -1313,7 +1313,7 @@ shdepReassignOwned(List *roleids, Oid newrole)
 			ereport(ERROR,
 					(errcode(ERRCODE_DEPENDENT_OBJECTS_STILL_EXIST),
 					 errmsg("cannot reassign ownership of objects owned by %s because they are required by the database system",
-							getObjectDescription(&obj))));
+							getObjectDescription(&obj, false))));
 
 			/*
 			 * There's no need to tell the whole truth, which is that we
diff --git a/src/backend/commands/event_trigger.c b/src/backend/commands/event_trigger.c
index fa7d0d015a..08e3035c14 100644
--- a/src/backend/commands/event_trigger.c
+++ b/src/backend/commands/event_trigger.c
@@ -1428,10 +1428,11 @@ EventTriggerSQLDropAddObject(const ObjectAddress *object, bool original, bool no
 
 	/* object identity, objname and objargs */
 	obj->objidentity =
-		getObjectIdentityParts(&obj->address, &obj->addrnames, &obj->addrargs);
+		getObjectIdentityParts(&obj->address, &obj->addrnames, &obj->addrargs,
+							   false);
 
 	/* object type */
-	obj->objecttype = getObjectTypeDescription(&obj->address);
+	obj->objecttype = getObjectTypeDescription(&obj->address, false);
 
 	slist_push_head(&(currentEventTriggerState->SQLDropList), &obj->next);
 
@@ -2089,8 +2090,8 @@ pg_event_trigger_ddl_commands(PG_FUNCTION_ARGS)
 					else if (cmd->type == SCT_AlterTSConfig)
 						addr = cmd->d.atscfg.address;
 
-					type = getObjectTypeDescription(&addr);
-					identity = getObjectIdentity(&addr);
+					type = getObjectTypeDescription(&addr, false);
+					identity = getObjectIdentity(&addr, false);
 
 					/*
 					 * Obtain schema name, if any ("pg_temp" if a temp
diff --git a/src/backend/commands/extension.c b/src/backend/commands/extension.c
index 9f77d25352..20f2c953c1 100644
--- a/src/backend/commands/extension.c
+++ b/src/backend/commands/extension.c
@@ -2826,7 +2826,7 @@ AlterExtensionNamespace(const char *extensionName, const char *newschema, Oid *o
 					 errmsg("extension \"%s\" does not support SET SCHEMA",
 							NameStr(extForm->extname)),
 					 errdetail("%s is not in the extension's schema \"%s\"",
-							   getObjectDescription(&dep),
+							   getObjectDescription(&dep, false),
 							   get_namespace_name(oldNspOid))));
 	}
 
@@ -3216,7 +3216,7 @@ ExecAlterExtensionContentsStmt(AlterExtensionContentsStmt *stmt,
 			ereport(ERROR,
 					(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
 					 errmsg("%s is already a member of extension \"%s\"",
-							getObjectDescription(&object),
+							getObjectDescription(&object, false),
 							get_extension_name(oldExtension))));
 
 		/*
@@ -3256,7 +3256,7 @@ ExecAlterExtensionContentsStmt(AlterExtensionContentsStmt *stmt,
 			ereport(ERROR,
 					(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
 					 errmsg("%s is not a member of extension \"%s\"",
-							getObjectDescription(&object),
+							getObjectDescription(&object, false),
 							stmt->extname)));
 
 		/*
diff --git a/src/backend/commands/tablecmds.c b/src/backend/commands/tablecmds.c
index 3971532447..896a8771d6 100644
--- a/src/backend/commands/tablecmds.c
+++ b/src/backend/commands/tablecmds.c
@@ -9221,7 +9221,7 @@ ATExecAlterColumnType(AlteredTableInfo *tab, Relation rel,
 					{
 						/* Not expecting any other direct dependencies... */
 						elog(ERROR, "unexpected object depending on column: %s",
-							 getObjectDescription(&foundObject));
+							 getObjectDescription(&foundObject, false));
 					}
 					break;
 				}
@@ -9269,7 +9269,7 @@ ATExecAlterColumnType(AlteredTableInfo *tab, Relation rel,
 						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 						 errmsg("cannot alter type of a column used by a view or rule"),
 						 errdetail("%s depends on column \"%s\"",
-								   getObjectDescription(&foundObject),
+								   getObjectDescription(&foundObject, false),
 								   colName)));
 				break;
 
@@ -9288,7 +9288,7 @@ ATExecAlterColumnType(AlteredTableInfo *tab, Relation rel,
 						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 						 errmsg("cannot alter type of a column used in a trigger definition"),
 						 errdetail("%s depends on column \"%s\"",
-								   getObjectDescription(&foundObject),
+								   getObjectDescription(&foundObject, false),
 								   colName)));
 				break;
 
@@ -9306,7 +9306,7 @@ ATExecAlterColumnType(AlteredTableInfo *tab, Relation rel,
 						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 						 errmsg("cannot alter type of a column used in a policy definition"),
 						 errdetail("%s depends on column \"%s\"",
-								   getObjectDescription(&foundObject),
+								   getObjectDescription(&foundObject, false),
 								   colName)));
 				break;
 
@@ -9367,7 +9367,7 @@ ATExecAlterColumnType(AlteredTableInfo *tab, Relation rel,
 				 * a column.
 				 */
 				elog(ERROR, "unexpected object depending on column: %s",
-					 getObjectDescription(&foundObject));
+					 getObjectDescription(&foundObject, false));
 				break;
 
 				/*
diff --git a/src/backend/utils/adt/format_type.c b/src/backend/utils/adt/format_type.c
index 00bfaca59e..08ab17470b 100644
--- a/src/backend/utils/adt/format_type.c
+++ b/src/backend/utils/adt/format_type.c
@@ -28,9 +28,6 @@
 
 #define MAX_INT32_LEN 11
 
-static char *format_type_internal(Oid type_oid, int32 typemod,
-					 bool typemod_given, bool allow_invalid,
-					 bool force_qualify);
 static char *printTypmod(const char *typname, int32 typmod, Oid typmodout);
 
 
@@ -74,11 +71,11 @@ format_type(PG_FUNCTION_ARGS)
 	type_oid = PG_GETARG_OID(0);
 
 	if (PG_ARGISNULL(1))
-		result = format_type_internal(type_oid, -1, false, true, false);
+		result = format_type_extended(type_oid, -1, false, true, false);
 	else
 	{
 		typemod = PG_GETARG_INT32(1);
-		result = format_type_internal(type_oid, typemod, true, true, false);
+		result = format_type_extended(type_oid, typemod, true, true, false);
 	}
 
 	PG_RETURN_TEXT_P(cstring_to_text(result));
@@ -93,7 +90,7 @@ format_type(PG_FUNCTION_ARGS)
 char *
 format_type_be(Oid type_oid)
 {
-	return format_type_internal(type_oid, -1, false, false, false);
+	return format_type_extended(type_oid, -1, false, false, false);
 }
 
 /*
@@ -103,7 +100,7 @@ format_type_be(Oid type_oid)
 char *
 format_type_be_qualified(Oid type_oid)
 {
-	return format_type_internal(type_oid, -1, false, false, true);
+	return format_type_extended(type_oid, -1, false, false, true);
 }
 
 /*
@@ -112,7 +109,7 @@ format_type_be_qualified(Oid type_oid)
 char *
 format_type_with_typemod(Oid type_oid, int32 typemod)
 {
-	return format_type_internal(type_oid, typemod, true, false, false);
+	return format_type_extended(type_oid, typemod, true, false, false);
 }
 
 /*
@@ -122,14 +119,14 @@ format_type_with_typemod(Oid type_oid, int32 typemod)
 char *
 format_type_with_typemod_qualified(Oid type_oid, int32 typemod)
 {
-	return format_type_internal(type_oid, typemod, true, false, true);
+	return format_type_extended(type_oid, typemod, true, false, true);
 }
 
 /*
  * Common workhorse.
  */
-static char *
-format_type_internal(Oid type_oid, int32 typemod,
+char *
+format_type_extended(Oid type_oid, int32 typemod,
 					 bool typemod_given, bool allow_invalid,
 					 bool force_qualify)
 {
@@ -141,7 +138,7 @@ format_type_internal(Oid type_oid, int32 typemod,
 	char	   *buf;
 
 	if (type_oid == InvalidOid && allow_invalid)
-		return pstrdup("-");
+			return pstrdup("-");
 
 	tuple = SearchSysCache1(TYPEOID, ObjectIdGetDatum(type_oid));
 	if (!HeapTupleIsValid(tuple))
@@ -437,7 +434,7 @@ oidvectortypes(PG_FUNCTION_ARGS)
 
 	for (num = 0; num < numargs; num++)
 	{
-		char	   *typename = format_type_internal(oidArray->values[num], -1,
+		char	   *typename = format_type_extended(oidArray->values[num], -1,
 													false, true, false);
 		size_t		slen = strlen(typename);
 
diff --git a/src/backend/utils/adt/regproc.c b/src/backend/utils/adt/regproc.c
index afd0c00b8a..2310a31b09 100644
--- a/src/backend/utils/adt/regproc.c
+++ b/src/backend/utils/adt/regproc.c
@@ -405,7 +405,8 @@ format_procedure_internal(Oid procedure_oid, bool force_qualify)
  * This can be used to feed get_object_address.
  */
 void
-format_procedure_parts(Oid procedure_oid, List **objnames, List **objargs)
+format_procedure_parts(Oid procedure_oid, List **objnames, List **objargs,
+					   bool missing_ok)
 {
 	HeapTuple	proctup;
 	Form_pg_proc procform;
@@ -415,7 +416,11 @@ format_procedure_parts(Oid procedure_oid, List **objnames, List **objargs)
 	proctup = SearchSysCache1(PROCOID, ObjectIdGetDatum(procedure_oid));
 
 	if (!HeapTupleIsValid(proctup))
+	{
+		if (missing_ok)
+			return;
 		elog(ERROR, "cache lookup failed for procedure with OID %u", procedure_oid);
+	}
 
 	procform = (Form_pg_proc) GETSTRUCT(proctup);
 	nargs = procform->pronargs;
@@ -751,7 +756,7 @@ to_regoperator(PG_FUNCTION_ARGS)
  * This exports the useful functionality of regoperatorout for use
  * in other backend modules.  The result is a palloc'd string.
  */
-static char *
+char *
 format_operator_internal(Oid operator_oid, bool force_qualify)
 {
 	char	   *result;
@@ -829,15 +834,20 @@ format_operator_qualified(Oid operator_oid)
 }
 
 void
-format_operator_parts(Oid operator_oid, List **objnames, List **objargs)
+format_operator_parts(Oid operator_oid, List **objnames, List **objargs,
+					  bool missing_ok)
 {
 	HeapTuple	opertup;
 	Form_pg_operator oprForm;
 
 	opertup = SearchSysCache1(OPEROID, ObjectIdGetDatum(operator_oid));
 	if (!HeapTupleIsValid(opertup))
+	{
+		if (missing_ok)
+			return;
 		elog(ERROR, "cache lookup failed for operator with OID %u",
 			 operator_oid);
+	}
 
 	oprForm = (Form_pg_operator) GETSTRUCT(opertup);
 	*objnames = list_make2(get_namespace_name_or_temp(oprForm->oprnamespace),
diff --git a/src/include/catalog/objectaddress.h b/src/include/catalog/objectaddress.h
index 5fc54d0e57..a8ab819d61 100644
--- a/src/include/catalog/objectaddress.h
+++ b/src/include/catalog/objectaddress.h
@@ -68,14 +68,18 @@ extern bool get_object_namensp_unique(Oid class_id);
 extern HeapTuple get_catalog_object_by_oid(Relation catalog,
 						  Oid objectId);
 
-extern char *getObjectDescription(const ObjectAddress *object);
+extern char *getObjectDescription(const ObjectAddress *object,
+								  bool missing_ok);
 extern char *getObjectDescriptionOids(Oid classid, Oid objid);
 
 extern int	read_objtype_from_string(const char *objtype);
-extern char *getObjectTypeDescription(const ObjectAddress *object);
-extern char *getObjectIdentity(const ObjectAddress *address);
+extern char *getObjectTypeDescription(const ObjectAddress *object,
+									  bool missing_ok);
+extern char *getObjectIdentity(const ObjectAddress *address,
+							   bool missing_ok);
 extern char *getObjectIdentityParts(const ObjectAddress *address,
-					   List **objname, List **objargs);
+									List **objname, List **objargs,
+									bool missing_ok);
 extern ArrayType *strlist_to_textarray(List *list);
 
 #endif							/* OBJECTADDRESS_H */
diff --git a/src/include/utils/builtins.h b/src/include/utils/builtins.h
index 762532f636..901a60f172 100644
--- a/src/include/utils/builtins.h
+++ b/src/include/utils/builtins.h
@@ -116,6 +116,9 @@ extern char *format_type_be(Oid type_oid);
 extern char *format_type_be_qualified(Oid type_oid);
 extern char *format_type_with_typemod(Oid type_oid, int32 typemod);
 extern char *format_type_with_typemod_qualified(Oid type_oid, int32 typemod);
+extern char *format_type_extended(Oid type_oid, int32 typemod,
+								  bool typemod_given, bool allow_invalid,
+								  bool force_qualify);
 extern int32 type_maximum_size(Oid type_oid, int32 typemod);
 
 /* quote.c */
diff --git a/src/include/utils/regproc.h b/src/include/utils/regproc.h
index ba46bd7d58..ed7d579242 100644
--- a/src/include/utils/regproc.h
+++ b/src/include/utils/regproc.h
@@ -19,10 +19,10 @@ extern List *stringToQualifiedNameList(const char *string);
 extern char *format_procedure(Oid procedure_oid);
 extern char *format_procedure_qualified(Oid procedure_oid);
 extern void format_procedure_parts(Oid operator_oid, List **objnames,
-					   List **objargs);
+								   List **objargs, bool missing_ok);
 extern char *format_operator(Oid operator_oid);
 extern char *format_operator_qualified(Oid operator_oid);
 extern void format_operator_parts(Oid operator_oid, List **objnames,
-					  List **objargs);
+								  List **objargs, bool missing_ok);
 
 #endif
diff --git a/src/test/regress/expected/object_address.out b/src/test/regress/expected/object_address.out
index 1fdadbc9ef..92e9f082bc 100644
--- a/src/test/regress/expected/object_address.out
+++ b/src/test/regress/expected/object_address.out
@@ -483,3 +483,711 @@ DROP SCHEMA addr_nsp CASCADE;
 NOTICE:  drop cascades to 12 other objects
 DROP OWNED BY regress_addr_user;
 DROP USER regress_addr_user;
+--
+-- Checks for invalid objects
+--
+-- Keep those checks in the same order as getObjectIdentityParts()
+SELECT * FROM pg_identify_object('pg_class'::regclass, 0, 0); -- no relation
+ type | schema | name | identity 
+------+--------+------+----------
+      |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_class'::regclass, 'pg_class'::regclass, -8); -- no column for relation
+     type     |   schema   |   name   |      identity       
+--------------+------------+----------+---------------------
+ table column | pg_catalog | pg_class | pg_catalog.pg_class
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_proc'::regclass, 0, 0); -- no function
+ type | schema | name | identity 
+------+--------+------+----------
+      |        |      | 0
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_type'::regclass, 0, 0); -- no type
+ type | schema | name | identity 
+------+--------+------+----------
+ type |        |      | -
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_cast'::regclass, 0, 0); -- no cast
+ type | schema | name | identity 
+------+--------+------+----------
+ cast |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_collation'::regclass, 0, 0); -- no collation
+   type    | schema | name | identity 
+-----------+--------+------+----------
+ collation |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_constraint'::regclass, 0, 0); -- no constraint
+ type | schema | name | identity 
+------+--------+------+----------
+      |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_conversion'::regclass, 0, 0); -- no conversion
+    type    | schema | name | identity 
+------------+--------+------+----------
+ conversion |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_attrdef'::regclass, 0, 0); -- no default attribute
+     type      | schema | name | identity 
+---------------+--------+------+----------
+ default value |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_language'::regclass, 0, 0); -- no language
+   type   | schema | name | identity 
+----------+--------+------+----------
+ language |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_largeobject'::regclass, 0, 0); -- no large object, no error
+     type     | schema | name | identity 
+--------------+--------+------+----------
+ large object |        |      | 0
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_operator'::regclass, 0, 0); -- no operator
+   type   | schema | name | identity 
+----------+--------+------+----------
+ operator |        |      | 0
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_opclass'::regclass, 0, 0); -- no opclass, no need to check for no access method
+      type      | schema | name | identity 
+----------------+--------+------+----------
+ operator class |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_opfamily'::regclass, 0, 0); -- no opfamily
+      type       | schema | name | identity 
+-----------------+--------+------+----------
+ operator family |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_am'::regclass, 0, 0); -- no access method
+     type      | schema | name | identity 
+---------------+--------+------+----------
+ access method |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_amop'::regclass, 0, 0); -- no AM operator
+           type            | schema | name | identity 
+---------------------------+--------+------+----------
+ operator of access method |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_amproc'::regclass, 0, 0); -- no AM proc
+           type            | schema | name | identity 
+---------------------------+--------+------+----------
+ function of access method |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_rewrite'::regclass, 0, 0); -- no rewrite
+ type | schema | name | identity 
+------+--------+------+----------
+ rule |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_trigger'::regclass, 0, 0); -- no trigger
+  type   | schema | name | identity 
+---------+--------+------+----------
+ trigger |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_namespace'::regclass, 0, 0); -- no schema
+  type  | schema | name | identity 
+--------+--------+------+----------
+ schema |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_statistic_ext'::regclass, 0, 0); -- no statistics
+       type        | schema | name | identity 
+-------------------+--------+------+----------
+ statistics object |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_ts_parser'::regclass, 0, 0); -- no TS parser
+        type        | schema | name | identity 
+--------------------+--------+------+----------
+ text search parser |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_ts_dict'::regclass, 0, 0); -- no TS dictionnary
+          type          | schema | name | identity 
+------------------------+--------+------+----------
+ text search dictionary |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_ts_template'::regclass, 0, 0); -- no TS template
+         type         | schema | name | identity 
+----------------------+--------+------+----------
+ text search template |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_ts_config'::regclass, 0, 0); -- no TS configuration
+           type            | schema | name | identity 
+---------------------------+--------+------+----------
+ text search configuration |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_authid'::regclass, 0, 0); -- no role
+ type | schema | name | identity 
+------+--------+------+----------
+ role |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_database'::regclass, 0, 0); -- no database
+   type   | schema | name | identity 
+----------+--------+------+----------
+ database |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_tablespace'::regclass, 0, 0); -- no tablespace
+    type    | schema | name | identity 
+------------+--------+------+----------
+ tablespace |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_foreign_data_wrapper'::regclass, 0, 0); -- no FDW
+         type         | schema | name | identity 
+----------------------+--------+------+----------
+ foreign-data wrapper |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_foreign_server'::regclass, 0, 0); -- no server
+  type  | schema | name | identity 
+--------+--------+------+----------
+ server |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_user_mapping'::regclass, 0, 0); -- no user mapping
+     type     | schema | name | identity 
+--------------+--------+------+----------
+ user mapping |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_default_acl'::regclass, 0, 0); -- no default ACL
+    type     | schema | name | identity 
+-------------+--------+------+----------
+ default acl |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_extension'::regclass, 0, 0); -- no extension
+   type    | schema | name | identity 
+-----------+--------+------+----------
+ extension |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_event_trigger'::regclass, 0, 0); -- no event trigger
+     type      | schema | name | identity 
+---------------+--------+------+----------
+ event trigger |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_policy'::regclass, 0, 0); -- no policy
+  type  | schema | name | identity 
+--------+--------+------+----------
+ policy |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_publication'::regclass, 0, 0); -- no publication
+    type     | schema | name | identity 
+-------------+--------+------+----------
+ publication |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_publication_rel'::regclass, 0, 0); -- no publication relation
+         type         | schema | name | identity 
+----------------------+--------+------+----------
+ publication relation |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_subscription'::regclass, 0, 0); -- no subscription
+     type     | schema | name | identity 
+--------------+--------+------+----------
+ subscription |        |      | 
+(1 row)
+
+SELECT * FROM pg_identify_object('pg_transform'::regclass, 0, 0); -- no transformation
+   type    | schema | name | identity 
+-----------+--------+------+----------
+ transform |        |      | 
+(1 row)
+
+-- Keep those checks in the same order as getObjectDescription()
+SELECT pg_describe_object('pg_class'::regclass, 0, 0); -- no relation
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_class'::regclass, 'pg_class'::regclass, -8); -- no column for relation
+ pg_describe_object 
+--------------------
+ table pg_class
+(1 row)
+
+SELECT pg_describe_object('pg_proc'::regclass, 0, 0); -- no function
+ pg_describe_object 
+--------------------
+ function 0
+(1 row)
+
+SELECT pg_describe_object('pg_type'::regclass, 0, 0); -- no type
+ pg_describe_object 
+--------------------
+ type -
+(1 row)
+
+SELECT pg_describe_object('pg_cast'::regclass, 0, 0); -- no cast
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_collation'::regclass, 0, 0); -- no collation
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_constraint'::regclass, 0, 0); -- no constraint
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_conversion'::regclass, 0, 0); -- no conversion
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_attrdef'::regclass, 0, 0); -- no default attribute
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_language'::regclass, 0, 0); -- no language
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_largeobject'::regclass, 0, 0); -- no large object, no error
+ pg_describe_object 
+--------------------
+ large object 0
+(1 row)
+
+SELECT pg_describe_object('pg_operator'::regclass, 0, 0); -- no operator
+ pg_describe_object 
+--------------------
+ operator 0
+(1 row)
+
+SELECT pg_describe_object('pg_opclass'::regclass, 0, 0); -- no opclass, no need to check for no access method
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_opfamily'::regclass, 0, 0); -- no opfamily
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_am'::regclass, 0, 0); -- no access method
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_amop'::regclass, 0, 0); -- no AM operator
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_amproc'::regclass, 0, 0); -- no AM proc
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_rewrite'::regclass, 0, 0); -- no rewrite
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_trigger'::regclass, 0, 0); -- no trigger
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_namespace'::regclass, 0, 0); -- no schema
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_statistic_ext'::regclass, 0, 0); -- no statistics
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_ts_parser'::regclass, 0, 0); -- no TS parser
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_ts_dict'::regclass, 0, 0); -- no TS dictionnary
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_ts_template'::regclass, 0, 0); -- no TS template
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_ts_config'::regclass, 0, 0); -- no TS configuration
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_authid'::regclass, 0, 0); -- no role
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_database'::regclass, 0, 0); -- no database
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_tablespace'::regclass, 0, 0); -- no tablespace
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_foreign_data_wrapper'::regclass, 0, 0); -- no FDW
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_foreign_server'::regclass, 0, 0); -- no server
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_user_mapping'::regclass, 0, 0); -- no user mapping
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_default_acl'::regclass, 0, 0); -- no default ACL
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_extension'::regclass, 0, 0); -- no extension
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_event_trigger'::regclass, 0, 0); -- no event trigger
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_policy'::regclass, 0, 0); -- no policy
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_publication'::regclass, 0, 0); -- no publication
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_publication_rel'::regclass, 0, 0); -- no publication relation
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_subscription'::regclass, 0, 0); -- no subscription
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+SELECT pg_describe_object('pg_transform'::regclass, 0, 0); -- no transformation
+ pg_describe_object 
+--------------------
+ 
+(1 row)
+
+-- Keep those checks in the same order as getObjectTypeDescription()
+SELECT * FROM pg_identify_object_as_address('pg_class'::regclass, 0, 0); -- no relation
+ type | object_names | object_args 
+------+--------------+-------------
+      |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_class'::regclass, 'pg_class'::regclass, -8); -- no column for relation
+     type     |     object_names      | object_args 
+--------------+-----------------------+-------------
+ table column | {pg_catalog,pg_class} | {}
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_proc'::regclass, 0, 0); -- no function
+ type | object_names | object_args 
+------+--------------+-------------
+      | {}           | {}
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_type'::regclass, 0, 0); -- no type
+ type | object_names | object_args 
+------+--------------+-------------
+ type | {-}          | {}
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_cast'::regclass, 0, 0); -- no cast
+ type | object_names | object_args 
+------+--------------+-------------
+ cast |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_collation'::regclass, 0, 0); -- no collation
+   type    | object_names | object_args 
+-----------+--------------+-------------
+ collation |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_constraint'::regclass, 0, 0); -- no constraint
+ type | object_names | object_args 
+------+--------------+-------------
+      |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_conversion'::regclass, 0, 0); -- no conversion
+    type    | object_names | object_args 
+------------+--------------+-------------
+ conversion |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_attrdef'::regclass, 0, 0); -- no default attribute
+     type      | object_names | object_args 
+---------------+--------------+-------------
+ default value |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_language'::regclass, 0, 0); -- no language
+   type   | object_names | object_args 
+----------+--------------+-------------
+ language |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_largeobject'::regclass, 0, 0); -- no large object, no error
+     type     | object_names | object_args 
+--------------+--------------+-------------
+ large object | {0}          | {}
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_operator'::regclass, 0, 0); -- no operator
+   type   | object_names | object_args 
+----------+--------------+-------------
+ operator | {}           | {}
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_opclass'::regclass, 0, 0); -- no opclass, no need to check for no access method
+      type      | object_names | object_args 
+----------------+--------------+-------------
+ operator class |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_opfamily'::regclass, 0, 0); -- no opfamily
+      type       | object_names | object_args 
+-----------------+--------------+-------------
+ operator family |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_am'::regclass, 0, 0); -- no access method
+     type      | object_names | object_args 
+---------------+--------------+-------------
+ access method |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_amop'::regclass, 0, 0); -- no AM operator
+           type            | object_names | object_args 
+---------------------------+--------------+-------------
+ operator of access method |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_amproc'::regclass, 0, 0); -- no AM proc
+           type            | object_names | object_args 
+---------------------------+--------------+-------------
+ function of access method |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_rewrite'::regclass, 0, 0); -- no rewrite
+ type | object_names | object_args 
+------+--------------+-------------
+ rule |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_trigger'::regclass, 0, 0); -- no trigger
+  type   | object_names | object_args 
+---------+--------------+-------------
+ trigger |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_namespace'::regclass, 0, 0); -- no schema
+  type  | object_names | object_args 
+--------+--------------+-------------
+ schema |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_statistic_ext'::regclass, 0, 0); -- no statistics
+       type        | object_names | object_args 
+-------------------+--------------+-------------
+ statistics object |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_ts_parser'::regclass, 0, 0); -- no TS parser
+        type        | object_names | object_args 
+--------------------+--------------+-------------
+ text search parser |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_ts_dict'::regclass, 0, 0); -- no TS dictionnary
+          type          | object_names | object_args 
+------------------------+--------------+-------------
+ text search dictionary |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_ts_template'::regclass, 0, 0); -- no TS template
+         type         | object_names | object_args 
+----------------------+--------------+-------------
+ text search template |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_ts_config'::regclass, 0, 0); -- no TS configuration
+           type            | object_names | object_args 
+---------------------------+--------------+-------------
+ text search configuration |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_authid'::regclass, 0, 0); -- no role
+ type | object_names | object_args 
+------+--------------+-------------
+ role |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_database'::regclass, 0, 0); -- no database
+   type   | object_names | object_args 
+----------+--------------+-------------
+ database |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_tablespace'::regclass, 0, 0); -- no tablespace
+    type    | object_names | object_args 
+------------+--------------+-------------
+ tablespace |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_foreign_data_wrapper'::regclass, 0, 0); -- no FDW
+         type         | object_names | object_args 
+----------------------+--------------+-------------
+ foreign-data wrapper |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_foreign_server'::regclass, 0, 0); -- no server
+  type  | object_names | object_args 
+--------+--------------+-------------
+ server |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_user_mapping'::regclass, 0, 0); -- no user mapping
+     type     | object_names | object_args 
+--------------+--------------+-------------
+ user mapping |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_default_acl'::regclass, 0, 0); -- no default ACL
+    type     | object_names | object_args 
+-------------+--------------+-------------
+ default acl |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_extension'::regclass, 0, 0); -- no extension
+   type    | object_names | object_args 
+-----------+--------------+-------------
+ extension |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_event_trigger'::regclass, 0, 0); -- no event trigger
+     type      | object_names | object_args 
+---------------+--------------+-------------
+ event trigger |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_policy'::regclass, 0, 0); -- no policy
+  type  | object_names | object_args 
+--------+--------------+-------------
+ policy |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_publication'::regclass, 0, 0); -- no publication
+    type     | object_names | object_args 
+-------------+--------------+-------------
+ publication |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_publication_rel'::regclass, 0, 0); -- no publication relation
+         type         | object_names | object_args 
+----------------------+--------------+-------------
+ publication relation |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_subscription'::regclass, 0, 0); -- no subscription
+     type     | object_names | object_args 
+--------------+--------------+-------------
+ subscription |              | 
+(1 row)
+
+SELECT * FROM pg_identify_object_as_address('pg_transform'::regclass, 0, 0); -- no transformation
+   type    | object_names | object_args 
+-----------+--------------+-------------
+ transform |              | 
+(1 row)
+
diff --git a/src/test/regress/sql/object_address.sql b/src/test/regress/sql/object_address.sql
index 63821b8008..e45af4bb08 100644
--- a/src/test/regress/sql/object_address.sql
+++ b/src/test/regress/sql/object_address.sql
@@ -211,3 +211,127 @@ DROP SCHEMA addr_nsp CASCADE;
 
 DROP OWNED BY regress_addr_user;
 DROP USER regress_addr_user;
+
+--
+-- Checks for invalid objects
+--
+-- Keep those checks in the same order as getObjectIdentityParts()
+SELECT * FROM pg_identify_object('pg_class'::regclass, 0, 0); -- no relation
+SELECT * FROM pg_identify_object('pg_class'::regclass, 'pg_class'::regclass, -8); -- no column for relation
+SELECT * FROM pg_identify_object('pg_proc'::regclass, 0, 0); -- no function
+SELECT * FROM pg_identify_object('pg_type'::regclass, 0, 0); -- no type
+SELECT * FROM pg_identify_object('pg_cast'::regclass, 0, 0); -- no cast
+SELECT * FROM pg_identify_object('pg_collation'::regclass, 0, 0); -- no collation
+SELECT * FROM pg_identify_object('pg_constraint'::regclass, 0, 0); -- no constraint
+SELECT * FROM pg_identify_object('pg_conversion'::regclass, 0, 0); -- no conversion
+SELECT * FROM pg_identify_object('pg_attrdef'::regclass, 0, 0); -- no default attribute
+SELECT * FROM pg_identify_object('pg_language'::regclass, 0, 0); -- no language
+SELECT * FROM pg_identify_object('pg_largeobject'::regclass, 0, 0); -- no large object, no error
+SELECT * FROM pg_identify_object('pg_operator'::regclass, 0, 0); -- no operator
+SELECT * FROM pg_identify_object('pg_opclass'::regclass, 0, 0); -- no opclass, no need to check for no access method
+SELECT * FROM pg_identify_object('pg_opfamily'::regclass, 0, 0); -- no opfamily
+SELECT * FROM pg_identify_object('pg_am'::regclass, 0, 0); -- no access method
+SELECT * FROM pg_identify_object('pg_amop'::regclass, 0, 0); -- no AM operator
+SELECT * FROM pg_identify_object('pg_amproc'::regclass, 0, 0); -- no AM proc
+SELECT * FROM pg_identify_object('pg_rewrite'::regclass, 0, 0); -- no rewrite
+SELECT * FROM pg_identify_object('pg_trigger'::regclass, 0, 0); -- no trigger
+SELECT * FROM pg_identify_object('pg_namespace'::regclass, 0, 0); -- no schema
+SELECT * FROM pg_identify_object('pg_statistic_ext'::regclass, 0, 0); -- no statistics
+SELECT * FROM pg_identify_object('pg_ts_parser'::regclass, 0, 0); -- no TS parser
+SELECT * FROM pg_identify_object('pg_ts_dict'::regclass, 0, 0); -- no TS dictionnary
+SELECT * FROM pg_identify_object('pg_ts_template'::regclass, 0, 0); -- no TS template
+SELECT * FROM pg_identify_object('pg_ts_config'::regclass, 0, 0); -- no TS configuration
+SELECT * FROM pg_identify_object('pg_authid'::regclass, 0, 0); -- no role
+SELECT * FROM pg_identify_object('pg_database'::regclass, 0, 0); -- no database
+SELECT * FROM pg_identify_object('pg_tablespace'::regclass, 0, 0); -- no tablespace
+SELECT * FROM pg_identify_object('pg_foreign_data_wrapper'::regclass, 0, 0); -- no FDW
+SELECT * FROM pg_identify_object('pg_foreign_server'::regclass, 0, 0); -- no server
+SELECT * FROM pg_identify_object('pg_user_mapping'::regclass, 0, 0); -- no user mapping
+SELECT * FROM pg_identify_object('pg_default_acl'::regclass, 0, 0); -- no default ACL
+SELECT * FROM pg_identify_object('pg_extension'::regclass, 0, 0); -- no extension
+SELECT * FROM pg_identify_object('pg_event_trigger'::regclass, 0, 0); -- no event trigger
+SELECT * FROM pg_identify_object('pg_policy'::regclass, 0, 0); -- no policy
+SELECT * FROM pg_identify_object('pg_publication'::regclass, 0, 0); -- no publication
+SELECT * FROM pg_identify_object('pg_publication_rel'::regclass, 0, 0); -- no publication relation
+SELECT * FROM pg_identify_object('pg_subscription'::regclass, 0, 0); -- no subscription
+SELECT * FROM pg_identify_object('pg_transform'::regclass, 0, 0); -- no transformation
+-- Keep those checks in the same order as getObjectDescription()
+SELECT pg_describe_object('pg_class'::regclass, 0, 0); -- no relation
+SELECT pg_describe_object('pg_class'::regclass, 'pg_class'::regclass, -8); -- no column for relation
+SELECT pg_describe_object('pg_proc'::regclass, 0, 0); -- no function
+SELECT pg_describe_object('pg_type'::regclass, 0, 0); -- no type
+SELECT pg_describe_object('pg_cast'::regclass, 0, 0); -- no cast
+SELECT pg_describe_object('pg_collation'::regclass, 0, 0); -- no collation
+SELECT pg_describe_object('pg_constraint'::regclass, 0, 0); -- no constraint
+SELECT pg_describe_object('pg_conversion'::regclass, 0, 0); -- no conversion
+SELECT pg_describe_object('pg_attrdef'::regclass, 0, 0); -- no default attribute
+SELECT pg_describe_object('pg_language'::regclass, 0, 0); -- no language
+SELECT pg_describe_object('pg_largeobject'::regclass, 0, 0); -- no large object, no error
+SELECT pg_describe_object('pg_operator'::regclass, 0, 0); -- no operator
+SELECT pg_describe_object('pg_opclass'::regclass, 0, 0); -- no opclass, no need to check for no access method
+SELECT pg_describe_object('pg_opfamily'::regclass, 0, 0); -- no opfamily
+SELECT pg_describe_object('pg_am'::regclass, 0, 0); -- no access method
+SELECT pg_describe_object('pg_amop'::regclass, 0, 0); -- no AM operator
+SELECT pg_describe_object('pg_amproc'::regclass, 0, 0); -- no AM proc
+SELECT pg_describe_object('pg_rewrite'::regclass, 0, 0); -- no rewrite
+SELECT pg_describe_object('pg_trigger'::regclass, 0, 0); -- no trigger
+SELECT pg_describe_object('pg_namespace'::regclass, 0, 0); -- no schema
+SELECT pg_describe_object('pg_statistic_ext'::regclass, 0, 0); -- no statistics
+SELECT pg_describe_object('pg_ts_parser'::regclass, 0, 0); -- no TS parser
+SELECT pg_describe_object('pg_ts_dict'::regclass, 0, 0); -- no TS dictionnary
+SELECT pg_describe_object('pg_ts_template'::regclass, 0, 0); -- no TS template
+SELECT pg_describe_object('pg_ts_config'::regclass, 0, 0); -- no TS configuration
+SELECT pg_describe_object('pg_authid'::regclass, 0, 0); -- no role
+SELECT pg_describe_object('pg_database'::regclass, 0, 0); -- no database
+SELECT pg_describe_object('pg_tablespace'::regclass, 0, 0); -- no tablespace
+SELECT pg_describe_object('pg_foreign_data_wrapper'::regclass, 0, 0); -- no FDW
+SELECT pg_describe_object('pg_foreign_server'::regclass, 0, 0); -- no server
+SELECT pg_describe_object('pg_user_mapping'::regclass, 0, 0); -- no user mapping
+SELECT pg_describe_object('pg_default_acl'::regclass, 0, 0); -- no default ACL
+SELECT pg_describe_object('pg_extension'::regclass, 0, 0); -- no extension
+SELECT pg_describe_object('pg_event_trigger'::regclass, 0, 0); -- no event trigger
+SELECT pg_describe_object('pg_policy'::regclass, 0, 0); -- no policy
+SELECT pg_describe_object('pg_publication'::regclass, 0, 0); -- no publication
+SELECT pg_describe_object('pg_publication_rel'::regclass, 0, 0); -- no publication relation
+SELECT pg_describe_object('pg_subscription'::regclass, 0, 0); -- no subscription
+SELECT pg_describe_object('pg_transform'::regclass, 0, 0); -- no transformation
+-- Keep those checks in the same order as getObjectTypeDescription()
+SELECT * FROM pg_identify_object_as_address('pg_class'::regclass, 0, 0); -- no relation
+SELECT * FROM pg_identify_object_as_address('pg_class'::regclass, 'pg_class'::regclass, -8); -- no column for relation
+SELECT * FROM pg_identify_object_as_address('pg_proc'::regclass, 0, 0); -- no function
+SELECT * FROM pg_identify_object_as_address('pg_type'::regclass, 0, 0); -- no type
+SELECT * FROM pg_identify_object_as_address('pg_cast'::regclass, 0, 0); -- no cast
+SELECT * FROM pg_identify_object_as_address('pg_collation'::regclass, 0, 0); -- no collation
+SELECT * FROM pg_identify_object_as_address('pg_constraint'::regclass, 0, 0); -- no constraint
+SELECT * FROM pg_identify_object_as_address('pg_conversion'::regclass, 0, 0); -- no conversion
+SELECT * FROM pg_identify_object_as_address('pg_attrdef'::regclass, 0, 0); -- no default attribute
+SELECT * FROM pg_identify_object_as_address('pg_language'::regclass, 0, 0); -- no language
+SELECT * FROM pg_identify_object_as_address('pg_largeobject'::regclass, 0, 0); -- no large object, no error
+SELECT * FROM pg_identify_object_as_address('pg_operator'::regclass, 0, 0); -- no operator
+SELECT * FROM pg_identify_object_as_address('pg_opclass'::regclass, 0, 0); -- no opclass, no need to check for no access method
+SELECT * FROM pg_identify_object_as_address('pg_opfamily'::regclass, 0, 0); -- no opfamily
+SELECT * FROM pg_identify_object_as_address('pg_am'::regclass, 0, 0); -- no access method
+SELECT * FROM pg_identify_object_as_address('pg_amop'::regclass, 0, 0); -- no AM operator
+SELECT * FROM pg_identify_object_as_address('pg_amproc'::regclass, 0, 0); -- no AM proc
+SELECT * FROM pg_identify_object_as_address('pg_rewrite'::regclass, 0, 0); -- no rewrite
+SELECT * FROM pg_identify_object_as_address('pg_trigger'::regclass, 0, 0); -- no trigger
+SELECT * FROM pg_identify_object_as_address('pg_namespace'::regclass, 0, 0); -- no schema
+SELECT * FROM pg_identify_object_as_address('pg_statistic_ext'::regclass, 0, 0); -- no statistics
+SELECT * FROM pg_identify_object_as_address('pg_ts_parser'::regclass, 0, 0); -- no TS parser
+SELECT * FROM pg_identify_object_as_address('pg_ts_dict'::regclass, 0, 0); -- no TS dictionnary
+SELECT * FROM pg_identify_object_as_address('pg_ts_template'::regclass, 0, 0); -- no TS template
+SELECT * FROM pg_identify_object_as_address('pg_ts_config'::regclass, 0, 0); -- no TS configuration
+SELECT * FROM pg_identify_object_as_address('pg_authid'::regclass, 0, 0); -- no role
+SELECT * FROM pg_identify_object_as_address('pg_database'::regclass, 0, 0); -- no database
+SELECT * FROM pg_identify_object_as_address('pg_tablespace'::regclass, 0, 0); -- no tablespace
+SELECT * FROM pg_identify_object_as_address('pg_foreign_data_wrapper'::regclass, 0, 0); -- no FDW
+SELECT * FROM pg_identify_object_as_address('pg_foreign_server'::regclass, 0, 0); -- no server
+SELECT * FROM pg_identify_object_as_address('pg_user_mapping'::regclass, 0, 0); -- no user mapping
+SELECT * FROM pg_identify_object_as_address('pg_default_acl'::regclass, 0, 0); -- no default ACL
+SELECT * FROM pg_identify_object_as_address('pg_extension'::regclass, 0, 0); -- no extension
+SELECT * FROM pg_identify_object_as_address('pg_event_trigger'::regclass, 0, 0); -- no event trigger
+SELECT * FROM pg_identify_object_as_address('pg_policy'::regclass, 0, 0); -- no policy
+SELECT * FROM pg_identify_object_as_address('pg_publication'::regclass, 0, 0); -- no publication
+SELECT * FROM pg_identify_object_as_address('pg_publication_rel'::regclass, 0, 0); -- no publication relation
+SELECT * FROM pg_identify_object_as_address('pg_subscription'::regclass, 0, 0); -- no subscription
+SELECT * FROM pg_identify_object_as_address('pg_transform'::regclass, 0, 0); -- no transformation
-- 
2.15.0

